if "sdist" in cmds:
    _sdist = cmds["sdist"]
elif "setuptools" in sys.modules:
    from setuptools.command.sdist import sdist as _sdist
else:
    from setuptools.command.sdist import sdist as _sdist
    from distutils.command.sdist import sdist as _sdist



"""Check that magic values are not used in comparisons"""

if 100 == 100: # [comparison-of-constants]
    pass

if 1 == 3: # [comparison-of-constants]
    pass

if 1 != 3: # [comparison-of-constants]
    pass

x = 0
if 4 == 3 == x: # [comparison-of-constants]
    pass

if x == 0: # correct
    pass

y = 1
if x == y: # correct
    pass

if 1 > 0: # [comparison-of-constants]
    pass

if x > 0: # correct
    pass

if 1 >= 0: # [comparison-of-constants]
    pass

if x >= 0: # correct
    pass

if 1 < 0: # [comparison-of-constants]
    pass

if x < 0: # correct
    pass

if 1 <= 0: # [comparison-of-constants]
    pass

if x <= 0: # correct
    pass

word = "hello"
if word == "": # correct
    pass

if "hello" == "": # [comparison-of-constants]
    pass

truthy = True
if truthy == True: # correct
    pass

if True == False: # [comparison-of-constants]
    pass



def f():
    items = [1, 2, 3, 4]
    result = []
    for i in items:
        result.append(i)  # PERF402


def f():
    items = [1, 2, 3, 4]
    result = []
    for i in items:
        result.insert(0, i)  # PERF402


def f():
    items = [1, 2, 3, 4]
    result = []
    for i in items:
        result.append(i * i)  # OK


def f():
    items = [1, 2, 3, 4]
    result = {}
    for i in items:
        result[i].append(i * i)  # OK


class Foo:
    def append(self, x):
        pass


def f():
    items = [1, 2, 3, 4]
    result = Foo()
    for i in items:
        result.append(i)  # OK


def f():
    import sys

    for path in ("foo", "bar"):
        sys.path.append(path)  # OK



from typing import (
    Union,
)

from typing_extensions import (
    TypeAlias,
)

TA0: TypeAlias = int
TA1: TypeAlias = int | float | bool
TA2: TypeAlias = Union[int, float, bool]


def good1(arg: int) -> int | bool:
    ...


def good2(arg: int, arg2: int | bool) -> None:
    ...


def f0(arg1: float | int) -> None:
    ...


def f1(arg1: float, *, arg2: float | list[str] | type[bool] | complex) -> None:
    ...


def f2(arg1: int, /, arg2: int | int | float) -> None:
    ...


def f3(arg1: int, *args: Union[int | int | float]) -> None:
    ...


async def f4(**kwargs: int | int | float) -> None:
    ...


class Foo:
    def good(self, arg: int) -> None:
        ...

    def bad(self, arg: int | float | complex) -> None:
        ...



def f():
    # These should both be ignored by the `noqa`.
    I = 1  # noqa: E741, F841


def f():
    # These should both be ignored by the `noqa`.
    I = 1  # noqa: E741,F841


def f():
    # These should both be ignored by the `noqa`.
    I = 1  # noqa: E741 F841


def f():
    # These should both be ignored by the `noqa`.
    I = 1  # noqa: E741 , F841


def f():
    # Only `E741` should be ignored by the `noqa`.
    I = 1  # noqa: E741.F841



class Str:
    def __str__(self):
        return 1

class Float:
    def __str__(self):
        return 3.05
    
class Int:
    def __str__(self):
        return 0
    
class Bool:
    def __str__(self):
        return False
    
class Str2:
    def __str__(self):
        x = "ruff"
        return x
    
# TODO fixme once Ruff has better type checking
def return_int():
    return 3

class ComplexReturn:
    def __str__(self):
        return return_int()


import collections
from collections import namedtuple
from typing import TypeAlias, TypeVar, NewType, NamedTuple, TypedDict

GLOBAL: str = "foo"


def assign():
    global GLOBAL
    GLOBAL = "bar"
    lower = 0
    Camel = 0
    CONSTANT = 0
    _ = 0

    MyObj1 = collections.namedtuple("MyObj1", ["a", "b"])
    MyObj2 = namedtuple("MyObj12", ["a", "b"])

    T = TypeVar("T")
    UserId = NewType("UserId", int)

    Employee = NamedTuple("Employee", [("name", str), ("id", int)])

    Point2D = TypedDict("Point2D", {"in": int, "x-y": int})

    IntOrStr: TypeAlias = int | str


def aug_assign(rank, world_size):
    global CURRENT_PORT

    CURRENT_PORT += 1
    if CURRENT_PORT > MAX_PORT:
        CURRENT_PORT = START_PORT


def loop_assign():
    global CURRENT_PORT
    for CURRENT_PORT in range(5):
        pass



# SIM109
if a == b or a == c:
    d

# SIM109
if (a == b or a == c) and None:
    d

# SIM109
if a == b or a == c or None:
    d

# SIM109
if a == b or None or a == c:
    d

# OK
if a in (b, c):
    d

# OK
if a == b or a == c():
    d

# OK
if (
    a == b
    # This comment prevents us from raising SIM109
    or a == c
):
    d



import logging
from distutils import log

from logging_setup import logger

logging.warn("Hello World!")
log.warn("Hello world!")  # This shouldn't be considered as a logger candidate
logger.warn("Hello world!")

logging . warn("Hello World!")



import collections

person: collections.namedtuple  # Y024 Use "typing.NamedTuple" instead of "collections.namedtuple"

from collections import namedtuple

person: namedtuple  # Y024 Use "typing.NamedTuple" instead of "collections.namedtuple"

person = namedtuple(
    "Person", ["name", "age"]
)  # Y024 Use "typing.NamedTuple" instead of "collections.namedtuple"



import sys
from sys import version_info

if sys.version_info >= (3, 4, 3): ...  # PYI004 
if sys.version_info < (3, 4, 3): ...  # PYI004 
if sys.version_info == (3, 4, 3): ...  # PYI004 
if sys.version_info != (3, 4, 3): ...  # PYI004 

if sys.version_info[0] == 2: ...
if version_info[0] == 2: ...
if sys.version_info < (3, 5): ...
if version_info >= (3, 5): ...
if sys.version_info[:2] == (2, 7): ...
if sys.version_info[:1] == (2,): ...
if sys.platform == 'linux': ...



import pytest


@pytest.mark.parametrize("x", [1, 1, 2])
def test_error_literal(x):
    ...


a = 1
b = 2
c = 3


@pytest.mark.parametrize("x", [a, a, b, b, b, c])
def test_error_expr_simple(x):
    ...


@pytest.mark.parametrize(
    "x",
    [
        (a, b),
        # comment
        (a, b),
        (b, c),
    ],
)
def test_error_expr_complex(x):
    ...


@pytest.mark.parametrize("x", [a, b, (a), c, ((a))])
def test_error_parentheses(x):
    ...


@pytest.mark.parametrize(
    "x",
    [
        a,
        b,
        (a),
        c,
        ((a)),
    ],
)
def test_error_parentheses_trailing_comma(x):
    ...


@pytest.mark.parametrize("x", [1, 2])
def test_ok(x):
    ...



def func(status):
    match status:
        case _:
            return 0
    return "unreachable"

def func(status):
    match status:
        case 1:
            return 1
    return 0

def func(status):
    match status:
        case 1:
            return 1
        case _:
            return 0

def func(status):
    match status:
        case 1 | 2 | 3:
            return 5
    return 6

def func(status):
    match status:
        case 1 | 2 | 3:
            return 5
        case _:
            return 10
    return 0

def func(status):
    match status:
        case 0:
            return 0
        case 1:
            return 1
        case 1:
            return "1 again"
        case _:
            return 3

def func(status):
    i = 0
    match status, i:
        case _, _:
            return 0

def func(status):
    i = 0
    match status, i:
        case _, 0:
            return 0
        case _, 2:
            return 0

def func(point):
    match point:
        case (0, 0):
            print("Origin")
        case _:
            raise ValueError("oops")

def func(point):
    match point:
        case (0, 0):
            print("Origin")
        case (0, y):
            print(f"Y={y}")
        case (x, 0):
            print(f"X={x}")
        case (x, y):
            print(f"X={x}, Y={y}")
        case _:
            raise ValueError("Not a point")

def where_is(point):
    class Point:
        x: int
        y: int

    match point:
        case Point(x=0, y=0):
            print("Origin")
        case Point(x=0, y=y):
            print(f"Y={y}")
        case Point(x=x, y=0):
            print(f"X={x}")
        case Point():
            print("Somewhere else")
        case _:
            print("Not a point")

def func(points):
    match points:
        case []:
            print("No points")
        case [Point(0, 0)]:
            print("The origin")
        case [Point(x, y)]:
            print(f"Single point {x}, {y}")
        case [Point(0, y1), Point(0, y2)]:
            print(f"Two on the Y axis at {y1}, {y2}")
        case _:
            print("Something else")

def func(point):
    match point:
        case Point(x, y) if x == y:
            print(f"Y=X at {x}")
        case Point(x, y):
            print(f"Not on the diagonal")

def func():
    from enum import Enum
    class Color(Enum):
        RED = 'red'
        GREEN = 'green'
        BLUE = 'blue'

    color = Color(input("Enter your choice of 'red', 'blue' or 'green': "))

    match color:
        case Color.RED:
            print("I see red!")
        case Color.GREEN:
            print("Grass is green")
        case Color.BLUE:
            print("I'm feeling the blues :(")



import typing
import sys
from typing import TypeAlias


_UnusedPrivateTypeAlias: TypeAlias = int | None
_T: typing.TypeAlias = str

# OK
_UsedPrivateTypeAlias: TypeAlias = int | None

def func(arg: _UsedPrivateTypeAlias) -> _UsedPrivateTypeAlias:
  ...


if sys.version_info > (3, 9):
  _PrivateTypeAlias: TypeAlias = str | None
else:
  _PrivateTypeAlias: TypeAlias = float | None


def func2(arg: _PrivateTypeAlias) -> None: ...



def func():
    while False:
        return "unreachable"
    return 1

def func():
    while False:
        return "unreachable"
    else:
        return 1

def func():
    while False:
        return "unreachable"
    else:
        return 1
    return "also unreachable"

def func():
    while True:
        return 1
    return "unreachable"

def func():
    while True:
        return 1
    else:
        return "unreachable"

def func():
    while True:
        return 1
    else:
        return "unreachable"
    return "also unreachable"

def func():
    i = 0
    while False:
        i += 1
    return i

def func():
    i = 0
    while True:
        i += 1
    return i

def func():
    while True:
        pass
    return 1

def func():
    i = 0
    while True:
        if True:
            print("ok")
        i += 1
    return i

def func():
    i = 0
    while True:
        if False:
            print("ok")
        i += 1
    return i

def func():
    while True:
        if True:
            return 1
    return 0

def func():
    while True:
        continue

def func():
    while False:
        continue

def func():
    while True:
        break

def func():
    while False:
        break

def func():
    while True:
        if True:
            continue

def func():
    while True:
        if True:
            break

'''
TODO: because `try` statements aren't handled this triggers a false positive as
the last statement is reached, but the rules thinks it isn't (it doesn't
see/process the break statement).

# Test case found in the Bokeh repository that trigger a false positive.
def bokeh2(self, host: str = DEFAULT_HOST, port: int = DEFAULT_PORT) -> None:
    self.stop_serving = False
    while True:
        try:
            self.server = HTTPServer((host, port), HtmlOnlyHandler)
            self.host = host
            self.port = port
            break
        except OSError:
            log.debug(f"port {port} is in use, trying to next one")
            port += 1

    self.thread = threading.Thread(target=self._run_web_server)
'''



"""
Should emit:
B030:
 - line 12, column 8
 - line 17, column 9
 - line 22, column 21
 - line 27, column 37
"""

try:
    pass
except 1:  # error
    pass

try:
    pass
except (1, ValueError):  # error
    pass

try:
    pass
except (ValueError, (RuntimeError, (KeyError, TypeError))):  # error
    pass

try:
    pass
except (ValueError, *(RuntimeError, (KeyError, TypeError))):  # error
    pass


try:
    pass
except (*a, *(RuntimeError, (KeyError, TypeError))):  # error
    pass

try:
    pass
except (ValueError, *(RuntimeError, TypeError)):  # ok
    pass

try:
    pass
except (ValueError, *[RuntimeError, *(TypeError,)]):  # ok
    pass


try:
    pass
except (*a, *b):  # ok
    pass


try:
    pass
except (*a, *(RuntimeError, TypeError)):  # ok
    pass


try:
    pass
except (*a, *(b, c)):  # ok
    pass


try:
    pass
except (*a, *(*b, *c)):  # ok
    pass


def what_to_catch():
    return ...


try:
    pass
except what_to_catch():  # ok
    pass



def foo():
    fruit = ["apple", "pear", "orange"]
    result = {}
    for idx, name in enumerate(fruit):
        result[idx] = name  # PERF403


def foo():
    fruit = ["apple", "pear", "orange"]
    result = {}
    for idx, name in enumerate(fruit):
        if idx % 2:
            result[idx] = name  # PERF403


def foo():
    fruit = ["apple", "pear", "orange"]
    result = {}
    for idx, name in enumerate(fruit):
        if idx % 2:
            result[idx] = name  # Ok (false negative: edge case where `else` is same as `if`)
        else:
            result[idx] = name


def foo():
    result = {}
    fruit = ["apple", "pear", "orange"]
    for idx, name in enumerate(fruit):
        if idx % 2:
            result[idx] = name  # PERF403


def foo():
    fruit = ["apple", "pear", "orange"]
    result = []
    for idx, name in enumerate(fruit):
        if idx % 2:
            result[idx] = name  # OK (result is not a dictionary)
        else:
            result[idx] = name


def foo():
    fruit = ["apple", "pear", "orange"]
    result = {}
    for idx, name in enumerate(fruit):
        if idx % 2:
            result[idx] = name  # OK (if/elif/else isn't replaceable)
        elif idx % 3:
            result[idx] = name
        else:
            result[idx] = name


def foo():
    result = {1: "banana"}
    fruit = ["apple", "pear", "orange"]
    for idx, name in enumerate(fruit):
        if idx % 2:
            result[idx] = name  # PERF403


def foo():
    fruit = ["apple", "pear", "orange"]
    result = {}
    for idx, name in enumerate(fruit):
        if idx in result:
            result[idx] = name  # PERF403


def foo():
    fruit = ["apple", "pear", "orange"]
    result = {}
    for name in fruit:
        result[name] = name  # PERF403


def foo():
    fruit = ["apple", "pear", "orange"]
    result = {}
    for idx, name in enumerate(fruit):
        result[name] = idx  # PERF403



from logging.config import BaseConfigurator  # banned
from typing import Any, Dict  # banned
from typing import *  # banned

from pandas import DataFrame  # banned
from pandas import *  # banned

import logging.config  # ok
import typing  # ok
import pandas  # ok



"""Tests to determine accurate detection of typing-only imports."""


def f():
    import pandas as pd  # TCH002

    x: pd.DataFrame


def f():
    from pandas import DataFrame  # TCH002

    x: DataFrame


def f():
    from pandas import DataFrame as df  # TCH002

    x: df


def f():
    import pandas as pd  # TCH002

    x: pd.DataFrame = 1


def f():
    from pandas import DataFrame  # TCH002

    x: DataFrame = 2


def f():
    from pandas import DataFrame as df  # TCH002

    x: df = 3


def f():
    import pandas as pd  # TCH002

    x: "pd.DataFrame" = 1


def f():
    import pandas as pd  # TCH002

    x = dict["pd.DataFrame", "pd.DataFrame"]


def f():
    import pandas as pd

    print(pd)


def f():
    from pandas import DataFrame

    print(DataFrame)


def f():
    from pandas import DataFrame

    def f():
        print(DataFrame)


def f():
    from typing import Dict, Any

    def example() -> Any:
        return 1

    x: Dict[int] = 20


def f():
    from typing import TYPE_CHECKING

    if TYPE_CHECKING:
        from typing import Dict
    x: Dict[int] = 20


def f():
    from pathlib import Path

    class ImportVisitor(ast.NodeTransformer):
        def __init__(self, cwd: Path) -> None:
            self.cwd = cwd
            origin = Path(spec.origin)

    class ExampleClass:
        def __init__(self):
            self.cwd = Path(pandas.getcwd())


def f():
    import pandas

    class Migration:
        enum = pandas


def f():
    import pandas

    class Migration:
        enum = pandas.EnumClass


def f():
    from typing import TYPE_CHECKING

    from pandas import y

    if TYPE_CHECKING:
        _type = x
    else:
        _type = y


def f():
    from typing import TYPE_CHECKING

    from pandas import y

    if TYPE_CHECKING:
        _type = x
    elif True:
        _type = y


def f():
    from typing import cast

    import pandas as pd

    x = cast(pd.DataFrame, 2)


def f():
    import pandas as pd

    x = dict[pd.DataFrame, pd.DataFrame]


def f():
    import pandas as pd


def f():
    from pandas import DataFrame  # noqa: TCH002

    x: DataFrame = 2


def f():
    from pandas import (  # noqa: TCH002
        DataFrame,
    )

    x: DataFrame = 2


def f():
    global Member

    from module import Member

    x: Member = 1



#!/usr/bin/env python3
"""Check two versions of ruff against a corpus of open-source code.

Example usage:

    scripts/check_ecosystem.py <path/to/ruff1> <path/to/ruff2>
"""
from __future__ import annotations

import argparse
import asyncio
import difflib
import heapq
import json
import logging
import re
import tempfile
import time
from asyncio.subprocess import PIPE, create_subprocess_exec
from contextlib import asynccontextmanager, nullcontext
from pathlib import Path
from signal import SIGINT, SIGTERM
from typing import TYPE_CHECKING, NamedTuple, Self, TypeVar

if TYPE_CHECKING:
    from collections.abc import AsyncIterator, Iterator, Sequence

logger = logging.getLogger(__name__)


class Repository(NamedTuple):
    """A GitHub repository at a specific ref."""

    org: str
    repo: str
    ref: str | None
    select: str = ""
    ignore: str = ""
    exclude: str = ""
    # Generating fixes is slow and verbose
    show_fixes: bool = False

    @asynccontextmanager
    async def clone(self: Self, checkout_dir: Path) -> AsyncIterator[Path]:
        """Shallow clone this repository to a temporary directory."""
        if checkout_dir.exists():
            logger.debug(f"Reusing {self.org}:{self.repo}")
            yield await self._get_commit(checkout_dir)
            return

        logger.debug(f"Cloning {self.org}:{self.repo}")
        git_clone_command = [
            "git",
            "clone",
            "--config",
            "advice.detachedHead=false",
            "--quiet",
            "--depth",
            "1",
            "--no-tags",
        ]
        if self.ref:
            git_clone_command.extend(["--branch", self.ref])

        git_clone_command.extend(
            [
                f"https://github.com/{self.org}/{self.repo}",
                checkout_dir,
            ],
        )

        git_clone_process = await create_subprocess_exec(
            *git_clone_command,
            env={"GIT_TERMINAL_PROMPT": "0"},
        )

        status_code = await git_clone_process.wait()

        logger.debug(
            f"Finished cloning {self.org}/{self.repo} with status {status_code}",
        )
        yield await self._get_commit(checkout_dir)

    def url_for(self: Self, commit_sha: str, path: str, lnum: int | None = None) -> str:
        """
        Return the GitHub URL for the given commit, path, and line number, if given.
        """
        # Default to main branch
        url = f"https://github.com/{self.org}/{self.repo}/blob/{commit_sha}/{path}"
        if lnum:
            url += f"#L{lnum}"
        return url

    async def _get_commit(self: Self, checkout_dir: Path) -> str:
        """Return the commit sha for the repository in the checkout directory."""
        git_sha_process = await create_subprocess_exec(
            *["git", "rev-parse", "HEAD"],
            cwd=checkout_dir,
            stdout=PIPE,
        )
        git_sha_stdout, _ = await git_sha_process.communicate()
        assert (
            await git_sha_process.wait() == 0
        ), f"Failed to retrieve commit sha at {checkout_dir}"
        return git_sha_stdout.decode().strip()


# Repositories to check
# We check most repositories with the default ruleset instead of all rules to avoid
# noisy reports when new rules are added; see https://github.com/astral-sh/ruff/pull/3590
REPOSITORIES: list[Repository] = [
    Repository("DisnakeDev", "disnake", "master"),
    Repository("PostHog", "HouseWatch", "main"),
    Repository("RasaHQ", "rasa", "main"),
    Repository("Snowflake-Labs", "snowcli", "main"),
    Repository("aiven", "aiven-client", "main"),
    Repository("alteryx", "featuretools", "main"),
    Repository("apache", "airflow", "main", select="ALL"),
    Repository("aws", "aws-sam-cli", "develop"),
    Repository("bloomberg", "pytest-memray", "main"),
    Repository("bokeh", "bokeh", "branch-3.3", select="ALL"),
    Repository("commaai", "openpilot", "master"),
    Repository("demisto", "content", "master"),
    Repository("docker", "docker-py", "main"),
    Repository("freedomofpress", "securedrop", "develop"),
    Repository("fronzbot", "blinkpy", "dev"),
    Repository("ibis-project", "ibis", "master"),
    Repository("ing-bank", "probatus", "main"),
    Repository("jrnl-org", "jrnl", "develop"),
    Repository("latchbio", "latch", "main"),
    Repository("lnbits", "lnbits", "main"),
    Repository("milvus-io", "pymilvus", "master"),
    Repository("mlflow", "mlflow", "master"),
    Repository("model-bakers", "model_bakery", "main"),
    Repository("pandas-dev", "pandas", "main"),
    Repository("prefecthq", "prefect", "main"),
    Repository("pypa", "build", "main"),
    Repository("pypa", "cibuildwheel", "main"),
    Repository("pypa", "pip", "main"),
    Repository("pypa", "setuptools", "main"),
    Repository("python", "mypy", "master"),
    Repository("python", "typeshed", "main", select="PYI"),
    Repository("python-poetry", "poetry", "master"),
    Repository("reflex-dev", "reflex", "main"),
    Repository("rotki", "rotki", "develop"),
    Repository("scikit-build", "scikit-build", "main"),
    Repository("scikit-build", "scikit-build-core", "main"),
    Repository("sphinx-doc", "sphinx", "master"),
    Repository("spruceid", "siwe-py", "main"),
    Repository("tiangolo", "fastapi", "master"),
    Repository("yandex", "ch-backup", "main"),
    Repository("zulip", "zulip", "main", select="ALL"),
]

SUMMARY_LINE_RE = re.compile(r"^(Found \d+ error.*)|(.*potentially fixable with.*)$")


class RuffError(Exception):
    """An error reported by ruff."""


async def check(
    *,
    ruff: Path,
    path: Path,
    name: str,
    select: str = "",
    ignore: str = "",
    exclude: str = "",
    show_fixes: bool = False,
) -> Sequence[str]:
    """Run the given ruff binary against the specified path."""
    logger.debug(f"Checking {name} with {ruff}")
    ruff_args = ["check", "--no-cache", "--exit-zero"]
    if select:
        ruff_args.extend(["--select", select])
    if ignore:
        ruff_args.extend(["--ignore", ignore])
    if exclude:
        ruff_args.extend(["--exclude", exclude])
    if show_fixes:
        ruff_args.extend(["--show-fixes", "--ecosystem-ci"])

    start = time.time()
    proc = await create_subprocess_exec(
        ruff.absolute(),
        *ruff_args,
        ".",
        stdout=PIPE,
        stderr=PIPE,
        cwd=path,
    )
    result, err = await proc.communicate()
    end = time.time()

    logger.debug(f"Finished checking {name} with {ruff} in {end - start:.2f}")

    if proc.returncode != 0:
        raise RuffError(err.decode("utf8"))

    lines = [
        line
        for line in result.decode("utf8").splitlines()
        if not SUMMARY_LINE_RE.match(line)
    ]

    return sorted(lines)


class Diff(NamedTuple):
    """A diff between two runs of ruff."""

    removed: set[str]
    added: set[str]
    source_sha: str

    def __bool__(self: Self) -> bool:
        """Return true if this diff is non-empty."""
        return bool(self.removed or self.added)

    def __iter__(self: Self) -> Iterator[str]:
        """Iterate through the changed lines in diff format."""
        for line in heapq.merge(sorted(self.removed), sorted(self.added)):
            if line in self.removed:
                yield f"- {line}"
            else:
                yield f"+ {line}"


async def compare(
    ruff1: Path,
    ruff2: Path,
    repo: Repository,
    checkouts: Path | None = None,
) -> Diff | None:
    """Check a specific repository against two versions of ruff."""
    removed, added = set(), set()

    # By the default, the git clone are transient, but if the user provides a
    # directory for permanent storage we keep it there
    if checkouts:
        location_context = nullcontext(checkouts)
    else:
        location_context = tempfile.TemporaryDirectory()

    with location_context as checkout_parent:
        assert ":" not in repo.org
        assert ":" not in repo.repo
        checkout_dir = Path(checkout_parent).joinpath(f"{repo.org}:{repo.repo}")
        async with repo.clone(checkout_dir) as checkout_sha:
            try:
                async with asyncio.TaskGroup() as tg:
                    check1 = tg.create_task(
                        check(
                            ruff=ruff1,
                            path=checkout_dir,
                            name=f"{repo.org}/{repo.repo}",
                            select=repo.select,
                            ignore=repo.ignore,
                            exclude=repo.exclude,
                            show_fixes=repo.show_fixes,
                        ),
                    )
                    check2 = tg.create_task(
                        check(
                            ruff=ruff2,
                            path=checkout_dir,
                            name=f"{repo.org}/{repo.repo}",
                            select=repo.select,
                            ignore=repo.ignore,
                            exclude=repo.exclude,
                            show_fixes=repo.show_fixes,
                        ),
                    )
            except ExceptionGroup as e:
                raise e.exceptions[0] from e

            for line in difflib.ndiff(check1.result(), check2.result()):
                if line.startswith("- "):
                    removed.add(line[2:])
                elif line.startswith("+ "):
                    added.add(line[2:])

    return Diff(removed, added, checkout_sha)


def read_projects_jsonl(projects_jsonl: Path) -> dict[tuple[str, str], Repository]:
    """Read either of the two formats of https://github.com/akx/ruff-usage-aggregate."""
    repositories = {}
    for line in projects_jsonl.read_text().splitlines():
        data = json.loads(line)
        # Check the input format.
        if "items" in data:
            for item in data["items"]:
                # Pick only the easier case for now.
                if item["path"] != "pyproject.toml":
                    continue
                repository = item["repository"]
                assert re.fullmatch(r"[a-zA-Z0-9_.-]+", repository["name"]), repository[
                    "name"
                ]
                # GitHub doesn't give us any branch or pure rev info.  This would give
                # us the revision, but there's no way with git to just do
                # `git clone --depth 1` with a specific ref.
                # `ref = item["url"].split("?ref=")[1]` would be exact
                repositories[(repository["owner"], repository["repo"])] = Repository(
                    repository["owner"]["login"],
                    repository["name"],
                    None,
                    select=repository.get("select"),
                    ignore=repository.get("ignore"),
                    exclude=repository.get("exclude"),
                )
        else:
            assert "owner" in data, "Unknown ruff-usage-aggregate format"
            # Pick only the easier case for now.
            if data["path"] != "pyproject.toml":
                continue
            repositories[(data["owner"], data["repo"])] = Repository(
                data["owner"],
                data["repo"],
                data.get("ref"),
                select=data.get("select"),
                ignore=data.get("ignore"),
                exclude=data.get("exclude"),
            )
    return repositories


DIFF_LINE_RE = re.compile(
    r"^(?P<pre>[+-]) (?P<inner>(?P<path>[^:]+):(?P<lnum>\d+):\d+:) (?P<post>.*)$",
)

T = TypeVar("T")


async def main(
    *,
    ruff1: Path,
    ruff2: Path,
    projects_jsonl: Path | None,
    checkouts: Path | None = None,
) -> None:
    """Check two versions of ruff against a corpus of open-source code."""
    if projects_jsonl:
        repositories = read_projects_jsonl(projects_jsonl)
    else:
        repositories = {(repo.org, repo.repo): repo for repo in REPOSITORIES}

    logger.debug(f"Checking {len(repositories)} projects")

    # https://stackoverflow.com/a/61478547/3549270
    # Otherwise doing 3k repositories can take >8GB RAM
    semaphore = asyncio.Semaphore(50)

    async def limited_parallelism(coroutine: T) -> T:
        async with semaphore:
            return await coroutine

    results = await asyncio.gather(
        *[
            limited_parallelism(compare(ruff1, ruff2, repo, checkouts))
            for repo in repositories.values()
        ],
        return_exceptions=True,
    )

    diffs = dict(zip(repositories, results, strict=True))

    total_removed = total_added = 0
    errors = 0

    for diff in diffs.values():
        if isinstance(diff, Exception):
            errors += 1
        else:
            total_removed += len(diff.removed)
            total_added += len(diff.added)

    if total_removed == 0 and total_added == 0 and errors == 0:
        print("\u2705 ecosystem check detected no changes.")
    else:
        rule_changes: dict[str, tuple[int, int]] = {}
        changes = f"(+{total_added}, -{total_removed}, {errors} error(s))"

        print(f"\u2139\ufe0f ecosystem check **detected changes**. {changes}")
        print()

        for (org, repo), diff in diffs.items():
            if isinstance(diff, Exception):
                changes = "error"
                print(f"<details><summary>{repo} ({changes})</summary>")
                repo = repositories[(org, repo)]
                print(
                    f"https://github.com/{repo.org}/{repo.repo} ref {repo.ref} "
                    f"select {repo.select} ignore {repo.ignore} exclude {repo.exclude}",
                )
                print("<p>")
                print()

                print("```")
                print(str(diff))
                print("```")

                print()
                print("</p>")
                print("</details>")
            elif diff:
                changes = f"+{len(diff.added)}, -{len(diff.removed)}"
                print(f"<details><summary>{repo} ({changes})</summary>")
                print("<p>")
                print()

                repo = repositories[(org, repo)]
                diff_lines = list(diff)

                print("<pre>")
                for line in diff_lines:
                    match = DIFF_LINE_RE.match(line)
                    if match is None:
                        print(line)
                        continue

                    pre, inner, path, lnum, post = match.groups()
                    url = repo.url_for(diff.source_sha, path, int(lnum))
                    print(f"{pre} <a href='{url}'>{inner}</a> {post}")
                print("</pre>")

                print()
                print("</p>")
                print("</details>")

                # Count rule changes
                for line in diff_lines:
                    # Find rule change for current line or construction
                    # + <rule>/<path>:<line>:<column>: <rule_code> <message>
                    matches = re.search(r": ([A-Z]{1,4}[0-9]{3,4})", line)

                    if matches is None:
                        # Handle case where there are no regex matches e.g.
                        # +                 "?application=AIRFLOW&authenticator=TEST_AUTH&role=TEST_ROLE&warehouse=TEST_WAREHOUSE" # noqa: E501, ERA001
                        # Which was found in local testing
                        continue

                    rule_code = matches.group(1)

                    # Get current additions and removals for this rule
                    current_changes = rule_changes.get(rule_code, (0, 0))

                    # Check if addition or removal depending on the first character
                    if line[0] == "+":
                        current_changes = (current_changes[0] + 1, current_changes[1])
                    elif line[0] == "-":
                        current_changes = (current_changes[0], current_changes[1] + 1)

                    rule_changes[rule_code] = current_changes

            else:
                continue

        if len(rule_changes.keys()) > 0:
            print(f"Rules changed: {len(rule_changes.keys())}")
            print()
            print("| Rule | Changes | Additions | Removals |")
            print("| ---- | ------- | --------- | -------- |")
            for rule, (additions, removals) in sorted(
                rule_changes.items(),
                key=lambda x: (x[1][0] + x[1][1]),
                reverse=True,
            ):
                print(f"| {rule} | {additions + removals} | {additions} | {removals} |")

    logger.debug(f"Finished {len(repositories)} repositories")


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Check two versions of ruff against a corpus of open-source code.",
        epilog="scripts/check_ecosystem.py <path/to/ruff1> <path/to/ruff2>",
    )

    parser.add_argument(
        "--projects",
        type=Path,
        help=(
            "Optional JSON files to use over the default repositories. "
            "Supports both github_search_*.jsonl and known-github-tomls.jsonl."
        ),
    )
    parser.add_argument(
        "--checkouts",
        type=Path,
        help=(
            "Location for the git checkouts, in case you want to save them"
            " (defaults to temporary directory)"
        ),
    )
    parser.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        help="Activate debug logging",
    )
    parser.add_argument(
        "ruff1",
        type=Path,
    )
    parser.add_argument(
        "ruff2",
        type=Path,
    )

    args = parser.parse_args()

    if args.verbose:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)

    loop = asyncio.get_event_loop()
    if args.checkouts:
        args.checkouts.mkdir(exist_ok=True, parents=True)
    main_task = asyncio.ensure_future(
        main(
            ruff1=args.ruff1,
            ruff2=args.ruff2,
            projects_jsonl=args.projects,
            checkouts=args.checkouts,
        ),
    )
    # https://stackoverflow.com/a/58840987/3549270
    for signal in [SIGINT, SIGTERM]:
        loop.add_signal_handler(signal, main_task.cancel)
    try:
        loop.run_until_complete(main_task)
    finally:
        loop.close()



from __future__ import annotations

from typing import Any

from requests import Session

from my_first_party import my_first_party_object

from . import my_local_folder_object











def main():
  my_local_folder_object.get()



class MyClass:
    ImportError = 4
    id: int
    dir = "/"

    def __init__(self):
        self.float = 5  # is fine
        self.id = 10
        self.dir = "."

    def str(self):
        pass


from typing import TypedDict


class MyClass(TypedDict):
    id: int


from threading import Event


class CustomEvent(Event):
    def set(self) -> None:
        ...

    def str(self) -> None:
        ...


from logging import Filter, LogRecord


class CustomFilter(Filter):
    def filter(self, record: LogRecord) -> bool:
        ...

    def str(self) -> None:
        ...


from typing_extensions import override


class MyClass:
    @override
    def str(self):
        pass

    def int(self):
        pass



from airflow.operators import PythonOperator


def my_callable():
    pass


my_task = PythonOperator(task_id="my_task", callable=my_callable)
my_task_2 = PythonOperator(callable=my_callable, task_id="my_task_2")

incorrect_name = PythonOperator(task_id="my_task")
incorrect_name_2 = PythonOperator(callable=my_callable, task_id="my_task_2")

from my_module import MyClass

incorrect_name = MyClass(task_id="my_task")



import a
# Don't take this comment into account when determining whether the next import can fit on one line.
from b import c
from d import e  # Do take this comment into account when determining whether the next import can fit on one line.
# The next import fits on one line.
from f import g  # 012ß9💣2ℝ9012ß9💣2ℝ9012ß9💣2ℝ9012ß9💣2ℝ9012ß9💣2ℝ9012ß9💣2ℝ9012ß9💣2ℝ
# The next import doesn't fit on one line.
from h import i  # 012ß9💣2ℝ9012ß9💣2ℝ9012ß9💣2ℝ9012ß9💣2ℝ9012ß9💣2ℝ9012ß9💣2ℝ9012ß9💣2ℝ9



# TDO006 - accepted
# TODO (evanrittenhouse): this is a valid TODO
# TDO006 - error
# ToDo (evanrittenhouse): invalid capitalization
# todo (evanrittenhouse): another invalid capitalization
# foo # todo: invalid capitalization



my_dict = {}
my_var = 0


def foo():
    my_var += 1


def bar():
    global my_var
    my_var += 1


def baz():
    global my_var
    global my_dict
    my_dict[my_var] += 1


def dec(x):
    return x


@dec
def f():
    dec = 1
    return dec


class Class:
    def f(self):
        print(my_var)
        my_var = 1


class Class:
    my_var = 0

    def f(self):
        print(my_var)
        my_var = 1


import sys


def main():
    print(sys.argv)

    try:
        3 / 0
    except ZeroDivisionError:
        import sys

        sys.exit(1)


import sys


def main():
    print(sys.argv)

    for sys in range(5):
        pass


import requests_mock as rm


def requests_mock(requests_mock: rm.Mocker):
    print(rm.ANY)


import sklearn.base
import mlflow.sklearn


def f():
    import sklearn

    mlflow



nonlocal x


def f():
    nonlocal x


def f():
    nonlocal y


def f():
    x = 1

    def f():
        nonlocal x

    def f():
        nonlocal y


def f():
    x = 1

    def g():
        nonlocal x

    del x


def f():
    def g():
        nonlocal x

    del x


def f():
    try:
        pass
    except Exception as x:
        pass

    def g():
        nonlocal x
        x = 2



from __future__ import annotations

import geopandas as gpd  # TCH002
import pydantic
import pyproj  # TCH002
from pydantic import BaseModel

import numpy


class A(BaseModel):
    x: pandas.DataFrame


class B(pydantic.BaseModel):
    x: numpy.ndarray


class C:
    pass


class D(C):
    x: pyproj.Transformer


class E(C):
    x: gpd.GeoDataFrame



###
# Non-fixable Errors.
###
foo + [  # This will be preserved.
]
[*foo] + [  # This will be preserved.
]
first = [
    # The order
    1,  # here
    2,  # is
    # extremely
    3,  # critical
    # to preserve
]
second = first + [
    # please
    4,
    # don't
    5,
    # touch
    6,
]


###
# Fixable errors.
###
class Fun:
    words = ("how", "fun!")

    def yay(self):
        return self.words


yay = Fun().yay

foo = [4, 5, 6]
bar = [1, 2, 3] + foo
zoob = tuple(bar)
quux = (7, 8, 9) + zoob
spam = quux + (10, 11, 12)
spom = list(spam)
eggs = spom + [13, 14, 15]
elatement = ("we all say",) + yay()
excitement = ("we all think",) + Fun().yay()
astonishment = ("we all feel",) + Fun.words

chain = ["a", "b", "c"] + eggs + list(("yes", "no", "pants") + zoob)

baz = () + zoob

[] + foo + [
]

pylint_call = [sys.executable, "-m", "pylint"] + args + [path]
pylint_call_tuple = (sys.executable, "-m", "pylint") + args + (path, path2)
b = a + [2, 3] + [4]

# Uses the non-preferred quote style, which should be retained.
f"{a() + ['b']}"

###
# Non-errors.
###
a = (1,) + [2]
a = [1, 2] + (3, 4)
a = ([1, 2, 3] + b) + (4, 5, 6)



import unittest
import pytest


class Test(unittest.TestCase):
    def test_pytest_raises(self):
        with pytest.raises(ValueError):
            raise ValueError

    def test_errors(self):
        with self.assertRaises(ValueError):
            raise ValueError



import builtins
from abc import abstractmethod

def __repr__(self) -> str: ...
def __str__(self) -> builtins.str: ...
def __repr__(self, /, foo) -> str: ...
def __repr__(self, *, foo) -> str: ...

class ShouldRemoveSingle:
    def __str__(self) -> builtins.str: ...  # Error: PYI029

class ShouldRemove:
    def __repr__(self) -> str: ...  # Error: PYI029
    def __str__(self) -> builtins.str: ...  # Error: PYI029

class NoReturnSpecified:
    def __str__(self): ...
    def __repr__(self): ...

class NonMatchingArgs:
    def __str__(self, *, extra) -> builtins.str: ...
    def __repr__(self, /, extra) -> str: ...

class MatchingArgsButAbstract:
    @abstractmethod
    def __str__(self) -> builtins.str: ...
    @abstractmethod
    def __repr__(self) -> str: ...



import pandas as pd

# Errors.
df = pd.read_table("data.csv", sep=",")
df = pd.read_table("data.csv", sep=",", header=0)
filename = "data.csv"
df = pd.read_table(filename, sep=",")
df = pd.read_table(filename, sep=",", header=0)

# Non-errors.
df = pd.read_csv("data.csv")
df = pd.read_table("data.tsv")
df = pd.read_table("data.tsv", sep="\t")
df = pd.read_table("data.tsv", sep=",,")
df = pd.read_table("data.tsv", sep=", ")
df = pd.read_table("data.tsv", sep=" ,")
df = pd.read_table("data.tsv", sep=" , ")
not_pd.read_table("data.csv", sep=",")
data = read_table("data.csv", sep=",")
data = read_table



"""
Violation:

Raising an exception using its assigned name is verbose and unrequired
"""
import logging

logger = logging.getLogger(__name__)


class MyException(Exception):
    pass


def bad():
    try:
        process()
    except MyException as e:
        logger.exception("process failed")
        raise e


def good():
    try:
        process()
    except MyException:
        logger.exception("process failed")
        raise


def still_good():
    try:
        process()
    except MyException as e:
        print(e)
        raise


def still_good_too():
    try:
        process()
    except MyException as e:
        print(e)
        raise e from None


def still_actually_good():
    try:
        process()
    except MyException as e:
        try:
            pass
        except TypeError:
            raise e


def bad_that_needs_recursion():
    try:
        process()
    except MyException as e:
        logger.exception("process failed")
        if True:
            raise e


def bad_that_needs_recursion_2():
    try:
        process()
    except MyException as e:
        logger.exception("process failed")
        if True:

            def foo():
                raise e



from typing import Dict, List, Optional, Set, Union, cast


def main() -> None:
    a_list: List[Optional[str]] = []
    a_list.append("hello")
    a_dict = cast(Dict[int | None, Union[int, Set[bool]]], {})
    a_dict[1] = {True, False}



from __future__ import annotations

import datetime
import pathlib
from uuid import UUID  # TCH003

import pydantic
from pydantic import BaseModel


class A(pydantic.BaseModel):
    x: datetime.datetime


class B(BaseModel):
    x: pathlib.Path


class C:
    pass


class D(C):
    x: UUID



###
# Errors.
###
def f():
    global X


def f():
    global X

    print(X)


###
# Non-errors.
###
def f():
    global X

    X = 1


def f():
    global X

    (X, y) = (1, 2)


def f():
    global X

    del X


def f():
    global X

    X += 1



from typing import (
    Union,
)

from typing_extensions import (
    TypeAlias,
)

# Type aliases not flagged
TA0: TypeAlias = int
TA1: TypeAlias = int | float | bool
TA2: TypeAlias = Union[int, float, bool]


def good1(arg: int) -> int | bool: ...


def good2(arg: int, arg2: int | bool) -> None: ...


def f0(arg1: float | int) -> None: ...  # PYI041


def f1(arg1: float, *, arg2: float | list[str] | type[bool] | complex) -> None: ...  # PYI041


def f2(arg1: int, /, arg2: int | int | float) -> None: ...  # PYI041


def f3(arg1: int, *args: Union[int | int | float]) -> None: ...  # PYI041


async def f4(**kwargs: int | int | float) -> None: ...  # PYI041


class Foo:
    def good(self, arg: int) -> None: ...

    def bad(self, arg: int | float | complex) -> None: ...  # PYI041



x = [2, 3, 1]
list(x)
list(sorted(x))
reversed(sorted(x))
reversed(sorted(x, key=lambda e: e))
reversed(sorted(x, reverse=True))
reversed(sorted(x, key=lambda e: e, reverse=True))
reversed(sorted(x, reverse=True, key=lambda e: e))
reversed(sorted(x, reverse=False))
reversed(sorted(x, reverse=x))
reversed(sorted(x, reverse=not x))

# Regression test for: https://github.com/astral-sh/ruff/issues/7289
reversed(sorted(i for i in range(42)))
reversed(sorted((i for i in range(42)), reverse=True))


def reversed(*args, **kwargs):
    return None


reversed(sorted(x, reverse=True))



def double_quotes_backslash():
    """Sum\\mary."""


def double_quotes_backslash_raw():
    r"""Sum\mary."""


def double_quotes_backslash_uppercase():
    R"""Sum\\mary."""


def make_unique_pod_id(pod_id: str) -> str | None:
    r"""
    Generate a unique Pod name.

    Kubernetes pod names must consist of one or more lowercase
    rfc1035/rfc1123 labels separated by '.' with a maximum length of 253
    characters.

    Name must pass the following regex for validation
    ``^[a-z0-9]([-a-z0-9]*[a-z0-9])?(\\.[a-z0-9]([-a-z0-9]*[a-z0-9])?)*$``

    For more details, see:
    https://github.com/kubernetes/kubernetes/blob/release-1.1/docs/design/identifiers.md

    :param pod_id: requested pod name
    :return: ``str`` valid Pod name of appropriate length
    """



#!/usr/bin/env python3
"""Here's a top-level ß9💣2ℝing that's over theß9💣2ℝ."""


def f1():
    """Here's a ß9💣2ℝing that's also over theß9💣2ℝ."""

    x = 1  # Here's a comment that's over theß9💣2ℝ, but it's not standalone.

    # Here's a standalone comment that's over theß9💣2ℝ.

    x = 2
    # Another standalone that is preceded by a newline and indent toke and is over theß9💣2ℝ.

    print("Here's a string that's over theß9💣2ℝ, but it's not a ß9💣2ℝing.")


"This is also considered a ß9💣2ℝing, and is over theß9💣2ℝ."


def f2():
    """Here's a multi-line ß9💣2ℝing.

    It's over theß9💣2ℝ on this line, which isn't the first line in the ß9💣2ℝing.
    """


def f3():
    """Here's a multi-line ß9💣2ℝing.

    It's over theß9💣2ℝ on this line, which isn't the first line in the ß9💣2ℝing."""



from typing import ParamSpec, TypeVar

# Errors.

T = TypeVar("T", covariant=True)
T = TypeVar("T", covariant=True, contravariant=False)
T = TypeVar("T", contravariant=True)
T = TypeVar("T", covariant=False, contravariant=True)
P = ParamSpec("P", covariant=True)
P = ParamSpec("P", covariant=True, contravariant=False)
P = ParamSpec("P", contravariant=True)
P = ParamSpec("P", covariant=False, contravariant=True)

T_co = TypeVar("T_co")
T_co = TypeVar("T_co", covariant=False)
T_co = TypeVar("T_co", contravariant=False)
T_co = TypeVar("T_co", covariant=False, contravariant=False)
T_co = TypeVar("T_co", contravariant=True)
T_co = TypeVar("T_co", covariant=False, contravariant=True)
P_co = ParamSpec("P_co")
P_co = ParamSpec("P_co", covariant=False)
P_co = ParamSpec("P_co", contravariant=False)
P_co = ParamSpec("P_co", covariant=False, contravariant=False)
P_co = ParamSpec("P_co", contravariant=True)
P_co = ParamSpec("P_co", covariant=False, contravariant=True)

T_contra = TypeVar("T_contra")
T_contra = TypeVar("T_contra", covariant=False)
T_contra = TypeVar("T_contra", contravariant=False)
T_contra = TypeVar("T_contra", covariant=False, contravariant=False)
T_contra = TypeVar("T_contra", covariant=True)
T_contra = TypeVar("T_contra", covariant=True, contravariant=False)
P_contra = ParamSpec("P_contra")
P_contra = ParamSpec("P_contra", covariant=False)
P_contra = ParamSpec("P_contra", contravariant=False)
P_contra = ParamSpec("P_contra", covariant=False, contravariant=False)
P_contra = ParamSpec("P_contra", covariant=True)
P_contra = ParamSpec("P_contra", covariant=True, contravariant=False)

# Non-errors.

T = TypeVar("T")
T = TypeVar("T", covariant=False)
T = TypeVar("T", contravariant=False)
T = TypeVar("T", covariant=False, contravariant=False)
P = ParamSpec("P")
P = ParamSpec("P", covariant=False)
P = ParamSpec("P", contravariant=False)
P = ParamSpec("P", covariant=False, contravariant=False)

T_co = TypeVar("T_co", covariant=True)
T_co = TypeVar("T_co", covariant=True, contravariant=False)
P_co = ParamSpec("P_co", covariant=True)
P_co = ParamSpec("P_co", covariant=True, contravariant=False)

T_contra = TypeVar("T_contra", contravariant=True)
T_contra = TypeVar("T_contra", covariant=False, contravariant=True)
P_contra = ParamSpec("P_contra", contravariant=True)
P_contra = ParamSpec("P_contra", covariant=False, contravariant=True)

# Bivariate types are errors, but not covered by this check.

T = TypeVar("T", covariant=True, contravariant=True)
P = ParamSpec("P", covariant=True, contravariant=True)
T_co = TypeVar("T_co", covariant=True, contravariant=True)
P_co = ParamSpec("P_co", covariant=True, contravariant=True)
T_contra = TypeVar("T_contra", covariant=True, contravariant=True)
P_contra = ParamSpec("P_contra", covariant=True, contravariant=True)



x = (
    "This"
    "is"
    "not"
)

x = (
    "This" \
    "is" \
    "not"
)

x = (
    "This"
    "is 'actually'"
    "fine"
)

x = (
    "This" \
    "is 'actually'" \
    "fine"
)

if True:
    "This can use 'double' quotes"
"But this needs to be changed"



"""Transform the README.md to support a specific deployment target.

By default, we assume that our README.md will be rendered on GitHub. However, different
targets have different strategies for rendering light- and dark-mode images. This script
adjusts the images in the README.md to support the given target.
"""
from __future__ import annotations

import argparse
from pathlib import Path

URL = "https://user-images.githubusercontent.com/1309177/{}.svg"
URL_LIGHT = URL.format("232603516-4fb4892d-585c-4b20-b810-3db9161831e4")
URL_DARK = URL.format("232603514-c95e9b0f-6b31-43de-9a80-9e844173fd6a")

# https://docs.github.com/en/get-started/writing-on-github/getting-started-with-writing-and-formatting-on-github/basic-writing-and-formatting-syntax#specifying-the-theme-an-image-is-shown-to
GITHUB = f"""
<p align="center">
  <picture align="center">
    <source media="(prefers-color-scheme: dark)" srcset="{URL_DARK}">
    <source media="(prefers-color-scheme: light)" srcset="{URL_LIGHT}">
    <img alt="Shows a bar chart with benchmark results." src="{URL_LIGHT}">
  </picture>
</p>
"""

# https://github.com/pypi/warehouse/issues/11251
PYPI = f"""
<p align="center">
  <img alt="Shows a bar chart with benchmark results." src="{URL_LIGHT}">
</p>
"""

# https://squidfunk.github.io/mkdocs-material/reference/images/#light-and-dark-mode
MK_DOCS = f"""
<p align="center">
  <img alt="Shows a bar chart with benchmark results." src="{URL_LIGHT}#only-light">
</p>

<p align="center">
  <img alt="Shows a bar chart with benchmark results." src="{URL_DARK}#only-dark">
</p>
"""


def main(target: str) -> None:
    """Modify the README.md to support the given target."""
    with Path("README.md").open(encoding="utf8") as fp:
        content = fp.read()
        if GITHUB not in content:
            msg = "README.md is not in the expected format."
            raise ValueError(msg)

    if target == "pypi":
        with Path("README.md").open("w", encoding="utf8") as fp:
            fp.write(content.replace(GITHUB, PYPI))
    elif target == "mkdocs":
        with Path("README.md").open("w", encoding="utf8") as fp:
            fp.write(content.replace(GITHUB, MK_DOCS))
    else:
        msg = f"Unknown target: {target}"
        raise ValueError(msg)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Modify the README.md to support a specific deployment target.",
    )
    parser.add_argument(
        "--target",
        type=str,
        required=True,
        choices=("pypi", "mkdocs"),
    )
    args = parser.parse_args()

    main(target=args.target)



from django.shortcuts import render


def test_view1(request):
    return render(request, "index.html", locals())


def test_view2(request):
    return render(request, "index.html", context=locals())


def test_view3(request):
    return render(request, "index.html")


def test_view4(request):
    return render(request, "index.html", {})


def test_view5(request):
    return render(request, "index.html", context={})



"""
Violation:

Checks for `raise` statements within `try` blocks.
"""
class MyException(Exception):
    pass


def bad():
    try:
        a = process()
        if not a:
            raise MyException(a)

        raise MyException(a)

        try:
            b = process()
            if not b:
                raise MyException(b)
        except Exception:
            logger.exception("something failed")
    except Exception:
        logger.exception("something failed")


def bad():
    try:
        a = process()
        if not a:
            raise MyException(a)

        raise MyException(a)

        try:
            b = process()
            if not b:
                raise MyException(b)
        except* Exception:
            logger.exception("something failed")
    except* Exception:
        logger.exception("something failed")


def good():
    try:
        a = process()  # This throws the exception now
    except MyException:
        logger.exception("a failed")
    except Exception:
        logger.exception("something failed")


def fine():
    try:
        a = process()  # This throws the exception now
    finally:
        print("finally")


def fine():
    try:
        raise ValueError("a doesn't exist")
    except TypeError: # A different exception is caught
        print("A different exception is caught")



import pandas as pd


data = pd.Series(range(1000))

# PD101
data.nunique() <= 1
data.nunique(dropna=True) <= 1
data.nunique(dropna=False) <= 1
data.nunique() == 1
data.nunique(dropna=True) == 1
data.nunique(dropna=False) == 1
data.nunique() != 1
data.nunique(dropna=True) != 1
data.nunique(dropna=False) != 1
data.nunique() > 1
data.dropna().nunique() == 1
data[data.notnull()].nunique() == 1

# No violation of this rule
data.nunique() == 0  # empty
data.nunique() >= 1  # not-empty
data.nunique() < 1  # empty
data.nunique() == 2  # not constant
data.unique() == 1  # not `nunique`

{"hello": "world"}.nunique() == 1  # no pd.Series



class OneAttributeClass:
    value: int
    ...


class OneAttributeClass2:
    ...
    value: int


class TwoEllipsesClass:
    ...
    ...


class DocstringClass:
    """
    My body only contains an ellipsis.
    """

    ...


class NonEmptyChild(Exception):
    value: int
    ...


class NonEmptyChild2(Exception):
    ...
    value: int


class NonEmptyWithInit:
    value: int
    ...

    def __init__():
        pass


class EmptyClass:
    ...


class EmptyEllipsis:
    ...


class Dog:
    eyes: int = 2


class WithInit:
    value: int = 0

    def __init__():
        ...


def function():
    ...


...



import pandas as pd

x = pd.DataFrame()

x.drop(["a"], axis=1, inplace=True)

x.y.drop(["a"], axis=1, inplace=True)

x["y"].drop(["a"], axis=1, inplace=True)

x.drop(
    inplace=True,
    columns=["a"],
    axis=1,
)

if True:
    x.drop(
        inplace=True,
        columns=["a"],
        axis=1,
    )

x.drop(["a"], axis=1, **kwargs, inplace=True)
x.drop(["a"], axis=1, inplace=True, **kwargs)
f(x.drop(["a"], axis=1, inplace=True))

x.apply(lambda x: x.sort_values("a", inplace=True))
import torch

torch.m.ReLU(inplace=True)  # safe because this isn't a pandas call

(x.drop(["a"], axis=1, inplace=True))



# Regression test for branch detection from
# https://github.com/pypa/build/blob/5800521541e5e749d4429617420d1ef8cdb40b46/src/build/_importlib.py
import sys

if sys.version_info < (3, 8):
    import importlib_metadata as metadata
elif sys.version_info < (3, 9, 10) or (3, 10, 0) <= sys.version_info < (3, 10, 2):
    try:
        import importlib_metadata as metadata
    except ModuleNotFoundError:
        from importlib import metadata
else:
    from importlib import metadata

__all__ = ["metadata"]



import typing
from collections.abc import Mapping
from typing import (
    Annotated,
    TypeAlias,
    Union,
    Literal,
)

_PrivateAliasT: TypeAlias = str | int  # PYI043, since this ends in a T
_PrivateAliasT2: TypeAlias = typing.Any  # PYI043, since this ends in a T
_PrivateAliasT3: TypeAlias = Literal[
    "not", "a", "chance"
]  # PYI043, since this ends in a T
just_literals_pipe_union: TypeAlias = Literal[True] | Literal["idk"]
PublicAliasT: TypeAlias = str | int
PublicAliasT2: TypeAlias = Union[str, bytes]
_ABCDEFGHIJKLMNOPQRST: TypeAlias = typing.Any
_PrivateAliasS: TypeAlias = Literal["I", "guess", "this", "is", "okay"]
_PrivateAliasS2: TypeAlias = Annotated[str, "also okay"]

# check that this edge case doesn't crash
_: TypeAlias = str | int



# Should change
foo, bar, baz = [fn(x) for x in items]

foo, bar, baz =[fn(x) for x in items]

foo, bar, baz =          [fn(x) for x in items]

foo, bar, baz = [[i for i in fn(x)] for x in items]

foo, bar, baz = [
    fn(x)
    for x in items
]

# Should not change
foo = [fn(x) for x in items]

x, = [await foo for foo in bar]



"""Test that `__all__` exports are respected even with multiple declarations."""

import random


def some_dependency_check():
    return random.uniform(0.0, 1.0) > 0.49999


if some_dependency_check():
    import math

    __all__ = ["math"]
else:
    __all__ = []



import builtins
import types
import typing
from collections.abc import Awaitable
from types import TracebackType
from typing import Any, Type

import _typeshed
import typing_extensions
from _typeshed import Unused

class GoodOne:
    def __exit__(self, *args: object) -> None: ...
    async def __aexit__(self, *args) -> str: ...

class GoodTwo:
    def __exit__(self, typ: type[builtins.BaseException] | None, *args: builtins.object) -> bool | None: ...
    async def __aexit__(self, /, typ: Type[BaseException] | None, *args: object, **kwargs) -> bool: ...

class GoodThree:
    def __exit__(self, __typ: typing.Type[BaseException] | None, exc: BaseException | None, *args: object) -> None: ...
    async def __aexit__(self, typ: typing_extensions.Type[BaseException] | None, __exc: BaseException | None, *args: object) -> None: ...

class GoodFour:
    def __exit__(self, typ: type[BaseException] | None, exc: BaseException | None, tb: TracebackType | None) -> None: ...
    async def __aexit__(self, typ: type[BaseException] | None, exc: BaseException | None, tb: types.TracebackType | None, *args: list[None]) -> None: ...

class GoodFive:
    def __exit__(self, typ: type[BaseException] | None, exc: BaseException | None, tb: TracebackType | None, weird_extra_arg: int = ..., *args: int, **kwargs: str) -> None: ...
    async def __aexit__(self, typ: type[BaseException] | None, exc: BaseException | None, tb: TracebackType | None) -> Awaitable[None]: ...

class GoodSix:
    def __exit__(self, typ: object, exc: builtins.object, tb: object) -> None: ...
    async def __aexit__(self, typ: object, exc: object, tb: builtins.object) -> None: ...

class GoodSeven:
    def __exit__(self, *args: Unused) -> bool: ...
    async def __aexit__(self, typ: Type[BaseException] | None, *args: _typeshed.Unused) -> Awaitable[None]: ...

class GoodEight:
    def __exit__(self, __typ: typing.Type[BaseException] | None, exc: BaseException | None, *args: _typeshed.Unused) -> bool: ...
    async def __aexit__(self, typ: type[BaseException] | None, exc: BaseException | None, tb: TracebackType | None, weird_extra_arg: int = ..., *args: Unused, **kwargs: Unused) -> Awaitable[None]: ...

class GoodNine:
    def __exit__(self, __typ: typing.Union[typing.Type[BaseException] , None], exc: typing.Union[BaseException , None], *args: _typeshed.Unused) -> bool: ...
    async def __aexit__(self, typ: typing.Union[typing.Type[BaseException], None], exc: typing.Union[BaseException , None], tb: typing.Union[TracebackType , None], weird_extra_arg: int = ..., *args: Unused, **kwargs: Unused) -> Awaitable[None]: ...

class GoodTen:
    def __exit__(self, __typ: typing.Optional[typing.Type[BaseException]], exc: typing.Optional[BaseException], *args: _typeshed.Unused) -> bool: ...
    async def __aexit__(self, typ: typing.Optional[typing.Type[BaseException]], exc: typing.Optional[BaseException], tb: typing.Optional[TracebackType], weird_extra_arg: int = ..., *args: Unused, **kwargs: Unused) -> Awaitable[None]: ...


class BadOne:
    def __exit__(self, *args: Any) -> None: ... # PYI036: Bad star-args annotation
    async def __aexit__(self) -> None: ... # PYI036: Missing args

class BadTwo:
    def __exit__(self, typ, exc, tb, weird_extra_arg) -> None: ... # PYI036: Extra arg must have default
    async def __aexit__(self, typ, exc, tb, *, weird_extra_arg) -> None: ...# PYI036: Extra arg must have default

class BadThree:
    def __exit__(self, typ: type[BaseException], exc: BaseException | None, tb: TracebackType | None) -> None: ... # PYI036: First arg has bad annotation
    async def __aexit__(self, __typ: type[BaseException] | None, __exc: BaseException, __tb: TracebackType) -> bool | None: ... # PYI036: Second arg has bad annotation

class BadFour:
    def __exit__(self, typ: typing.Optional[type[BaseException]], exc: typing.Union[BaseException, None], tb: TracebackType) -> None: ... # PYI036: Third arg has bad annotation
    async def __aexit__(self, __typ: type[BaseException] | None, __exc: BaseException | None, __tb: typing.Union[TracebackType, None, int]) -> bool | None: ... # PYI036: Third arg has bad annotation

class BadFive:
    def __exit__(self, typ: BaseException | None, *args: list[str]) -> bool: ... # PYI036: Bad star-args annotation
    async def __aexit__(self, /, typ: type[BaseException] | None, *args: Any) -> Awaitable[None]: ... # PYI036: Bad star-args annotation

class BadSix:
    def __exit__(self, typ, exc, tb, weird_extra_arg, extra_arg2 = None) -> None: ... # PYI036: Extra arg must have default
    async def __aexit__(self, typ, exc, tb, *, weird_extra_arg) -> None: ... # PYI036: kwargs must have default



# formfeed indent
# https://github.com/astral-sh/ruff/issues/7455#issuecomment-1722458825
# This is technically a stylist bug (and has a test there), but it surfaced in B006


class FormFeedIndent:
   def __init__(self, a=[]):
        print(a)




"""Generate the confusables.rs file from the VS Code ambiguous.json file."""
from __future__ import annotations

import json
import subprocess
from pathlib import Path

CONFUSABLES_RS_PATH = "crates/ruff_linter/src/rules/ruff/rules/confusables.rs"
AMBIGUOUS_JSON_URL = "https://raw.githubusercontent.com/hediet/vscode-unicode-data/main/out/ambiguous.json"

prelude = """
//! This file is auto-generated by `scripts/update_ambiguous_characters.py`.

/// Via: <https://github.com/hediet/vscode-unicode-data/blob/main/out/ambiguous.json>
/// See: <https://github.com/microsoft/vscode/blob/095ddabc52b82498ee7f718a34f9dd11d59099a8/src/vs/base/common/strings.ts#L1094>
pub(crate) fn confusable(c: u32) -> Option<u8> {
  let result = match c {

""".lstrip()

postlude = """_ => return None, }; Some(result)}"""


def get_mapping_data() -> dict:
    """
    Get the ambiguous character mapping data from the vscode-unicode-data repository.

    Uses the system's `curl` command to download the data,
    instead of adding a dependency to a Python-native HTTP client.
    """
    content = subprocess.check_output(
        ["curl", "-sSL", AMBIGUOUS_JSON_URL],
        encoding="utf-8",
    )
    # The content is a JSON object literal wrapped in a JSON string, so double decode:
    return json.loads(json.loads(content))


def format_confusables_rs(raw_data: dict[str, list[int]]) -> str:
    """Format the downloaded data into a Rust source file."""
    # The input data contains duplicate entries
    flattened_items: set[tuple[int, int]] = set()
    for _category, items in raw_data.items():
        assert len(items) % 2 == 0, "Expected pairs of items"
        for i in range(0, len(items), 2):
            flattened_items.add((items[i], items[i + 1]))

    tuples = [f"    {left}u32 => {right},\n" for left, right in sorted(flattened_items)]

    print(f"{len(tuples)} confusable tuples.")

    return prelude + "".join(tuples) + postlude


def main() -> None:
    print("Retrieving data...")
    mapping_data = get_mapping_data()
    formatted_data = format_confusables_rs(mapping_data)
    confusables_path = Path(__file__).parent.parent / CONFUSABLES_RS_PATH
    confusables_path.write_text(formatted_data, encoding="utf-8")
    print("Formatting Rust file with cargo fmt...")
    subprocess.check_call(["cargo", "fmt", "--", confusables_path])
    print("Done.")


if __name__ == "__main__":
    main()



import typing
from typing import cast

# For -> for, variable reused
for i in []:
    for i in []:  # error
        pass

# With -> for, variable reused
with None as i:
    for i in []:  # error
        pass

# For -> with, variable reused
for i in []:
    with None as i:  # error
        pass

# With -> with, variable reused
with None as i:
    with None as i:  # error
        pass

# For -> for, different variable
for i in []:
    for j in []:  # ok
        pass

# With -> with, different variable
with None as i:
    with None as j:  # ok
        pass

# Async with -> with, variable reused
async with None as i:
    with None as i:  # error
        pass

# Async with -> with, different variable
async with None as i:
    with None as j:  # ok
        pass

# Async for -> for, variable reused
async for i in []:
    for i in []:  # error
        pass

# For -> for -> for, doubly nested variable reuse
for i in []:
    for j in []:
        for i in []:  # error
            pass

# For -> for -> for -> for, doubly nested variable reuse x2
for i in []:
    for j in []:
        for i in []:  # error
            for j in []:  # error
                pass

# For -> assignment
for i in []:
    # ignore typing cast
    i = cast(int, i)
    i = typing.cast(int, i)
    i = 5  # error

# For -> augmented assignment
for i in []:
    i += 5  # error

# For -> annotated assignment
for i in []:
    i: int = 5  # error

# For -> annotated assignment without value
for i in []:
    i: int  # no error

# Async for -> for, variable reused
async for i in []:
    for i in []:  # error
        pass

# For -> async for, variable reused
for i in []:
    async for i in []:  # error
        pass

# For -> for, outer loop unpacks tuple
for i, j in enumerate([]):
    for i in []:  # error
        pass

# For -> for, inner loop unpacks tuple
for i in []:
    for i, j in enumerate([]):  # error
        pass

# For -> for, both loops unpack tuple
for (i, (j, k)) in []:
    for i, j in enumerate([]):  # two errors
        pass

# For else -> for, variable reused in else
for i in []:
    pass
else:
    for i in []:  # no error
        pass

# For -> for, ignore dummy variables
for _ in []:
    for _ in []:  # no error
        pass

# For -> for, outer loop unpacks with asterisk
for i, *j in []:
    for j in []:  # error
        pass

# For -> function definition
for i in []:
    def f():
        i = 2  # no error

# For -> class definition
for i in []:
    class A:
        i = 2  # no error

# For -> function definition -> for -> assignment
for i in []:
    def f():
        for i in []:  # no error
            i = 2  # error

# For -> class definition -> for -> for
for i in []:
    class A:
        for i in []:  # no error
            for i in []:  # error
                pass

# For -> use in assignment target without actual assignment; subscript
for i in []:
    a[i] = 2  # no error
    i[a] = 2  # no error

# For -> use in assignment target without actual assignment; attribute
for i in []:
    a.i = 2  # no error
    i.a = 2  # no error

# For target with subscript -> assignment
for a[0] in []:
    a[0] = 2  # error
    a[1] = 2  # no error

# For target with subscript -> assignment
for a['i'] in []:
    a['i'] = 2  # error
    a['j'] = 2  # no error

# For target with attribute -> assignment
for a.i in []:
    a.i = 2  # error
    a.j = 2  # no error

# For target with double nested attribute -> assignment
for a.i.j in []:
    a.i.j = 2  # error
    a.j.i = 2  # no error

# For target with attribute -> assignment with different spacing
for a.i in []:
    a. i = 2  # error
for a. i in []:
    a.i = 2  # error



def test_division():
    a = 9 / 3
    assert "No ZeroDivisionError were raised"  # [assert-on-string-literal]


def test_division():
    a = 9 / 3
    assert a == 3


try:
    assert "bad"  # [assert-on-string-literal]
except:
    assert "bad again"  # [assert-on-string-literal]

a = 12
assert f"hello {a}"  # [assert-on-string-literal]
assert f"{a}"  # [assert-on-string-literal]
assert f""  # [assert-on-string-literal]
assert ""  # [assert-on-string-literal]
assert b"hello"  # [assert-on-string-literal]
assert "", b"hi"  # [assert-on-string-literal]
assert "WhyNotHere?", "HereIsOk"  # [assert-on-string-literal]
assert 12, "ok here"



import builtins
import typing
from typing import TypeAlias, Final, NewType, TypeVar, TypeVarTuple, ParamSpec

# We shouldn't emit Y015 for simple default values
field1: int
field2: int = ...
field3 = ...  # type: int  # Y033 Do not use type comments in stubs (e.g. use "x: int" instead of "x = ... # type: int")
field4: int = 0
field41: int = 0xFFFFFFFF
field42: int = 1234567890
field43: int = -0xFFFFFFFF
field44: int = -1234567890
field5 = 0  # type: int  # Y033 Do not use type comments in stubs (e.g. use "x: int" instead of "x = ... # type: int")  # Y052 Need type annotation for "field5"
field6 = 0  # Y052 Need type annotation for "field6"
field7 = b""  # Y052 Need type annotation for "field7"
field71 = "foo"  # Y052 Need type annotation for "field71"
field72: str = "foo"
field8 = False  # Y052 Need type annotation for "field8"
field81 = -1  # Y052 Need type annotation for "field81"
field82: float = -98.43
field83 = -42j  # Y052 Need type annotation for "field83"
field84 = 5 + 42j  # Y052 Need type annotation for "field84"
field85 = -5 - 42j  # Y052 Need type annotation for "field85"
field9 = None  # Y026 Use typing_extensions.TypeAlias for type aliases, e.g. "field9: TypeAlias = None"
Field95: TypeAlias = None
Field96: TypeAlias = int | None
Field97: TypeAlias = None | typing.SupportsInt | builtins.str | float | bool
Field98 = NewType('MyInt', int)
Field99 = TypeVar('Field99')
Field100 = TypeVarTuple('Field100')
Field101 = ParamSpec('Field101')
field19 = [1, 2, 3]  # Y052 Need type annotation for "field19"
field191: list[int] = [1, 2, 3]
field20 = (1, 2, 3)  # Y052 Need type annotation for "field20"
field201: tuple[int, ...] = (1, 2, 3)
field21 = {1, 2, 3}  # Y052 Need type annotation for "field21"
field211: set[int] = {1, 2, 3}
field212 = {"foo": "bar"}  # Y052 Need type annotation for "field212"
field213: dict[str, str] = {"foo": "bar"}
field22: Final = {"foo": 5}

# We *should* emit Y015 for more complex default values
field221: list[int] = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]  # Y015 Only simple default values are allowed for assignments
field223: list[int] = [*range(10)]  # Y015 Only simple default values are allowed for assignments
field224: list[int] = list(range(10))  # Y015 Only simple default values are allowed for assignments
field225: list[object] = [{}, 1, 2]  # Y015 Only simple default values are allowed for assignments
field226: tuple[str | tuple[str, ...], ...] = ("foo", ("foo", "bar"))  # Y015 Only simple default values are allowed for assignments
field227: dict[str, object] = {"foo": {"foo": "bar"}}  # Y015 Only simple default values are allowed for assignments
field228: dict[str, list[object]] = {"foo": []}  # Y015 Only simple default values are allowed for assignments
# When parsed, this case results in `None` being placed in the `.keys` list for the `ast.Dict` node
field229: dict[int, int] = {1: 2, **{3: 4}}  # Y015 Only simple default values are allowed for assignments
field23 = "foo" + "bar"  # Y015 Only simple default values are allowed for assignments
field24 = b"foo" + b"bar"  # Y015 Only simple default values are allowed for assignments
field25 = 5 * 5  # Y015 Only simple default values are allowed for assignments

# We shouldn't emit Y015 within functions
def f():
  field26: list[int] = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]


# We shouldn't emit Y015 for __slots__ or __match_args__
class Class1:
  __slots__ = (
    '_one',
    '_two',
    '_three',
    '_four',
    '_five',
    '_six',
    '_seven',
    '_eight',
    '_nine',
    '_ten',
    '_eleven',
  )

  __match_args__ = (
    'one',
    'two',
    'three',
    'four',
    'five',
    'six',
    'seven',
    'eight',
    'nine',
    'ten',
    'eleven',
  )

# We shouldn't emit Y015 for __all__
__all__ = ["Class1"]

# Ignore the following for PYI015
field26 = typing.Sequence[int]
field27 = list[str]
field28 = builtins.str
field29 = str
field30 = str | bytes | None
field31: typing.Final = field30



# Errors
"yoda" == compare  # SIM300
"yoda" == compare  # SIM300
42 == age  # SIM300
("a", "b") == compare  # SIM300
"yoda" <= compare  # SIM300
"yoda" < compare  # SIM300
42 > age  # SIM300
-42 > age  # SIM300
+42 > age  # SIM300
YODA == age  # SIM300
YODA > age  # SIM300
YODA >= age  # SIM300
JediOrder.YODA == age  # SIM300
0 < (number - 100)  # SIM300
SomeClass().settings.SOME_CONSTANT_VALUE > (60 * 60)  # SIM300
B<A[0][0]or B
B or(B)<A[0][0]

# OK
compare == "yoda"
age == 42
compare == ("a", "b")
x == y
"yoda" == compare == 1
"yoda" == compare == someothervar
"yoda" == "yoda"
age == YODA
age < YODA
age <= YODA
YODA == YODA
age == JediOrder.YODA
(number - 100) > 0



import e
import f

# isort: split

import c
import d

# isort: split

import a
import b

if True:
    import C
    import A

    # isort: split

    import D
    import B


import e
import f

# isort: split
# isort: split

import d
import c



## Banned modules ##
import torch

from torch import *

from tensorflow import a, b, c

import torch as torch_wearing_a_trenchcoat

# this should count as module level
x = 1; import tensorflow

# banning a module also bans any submodules
import torch.foo.bar

from tensorflow.foo import bar

from torch.foo.bar import *

# unlike TID251, inline imports are *not* banned
def my_cool_function():
    import tensorflow.foo.bar

def another_cool_function():
    from torch.foo import bar

def import_alias():
    from torch.foo import bar

if TYPE_CHECKING:
    import torch



import math  # not checked

import altair  # unconventional
import dask.array  # unconventional
import dask.dataframe  # unconventional
import matplotlib.pyplot  # unconventional
import numpy  # unconventional
import pandas  # unconventional
import seaborn  # unconventional
import tensorflow  # unconventional
import holoviews  # unconventional
import panel  # unconventional
import plotly.express  # unconventional
import matplotlib  # unconventional
import polars  # unconventional
import pyarrow  # unconventional

import altair as altr  # unconventional
import matplotlib.pyplot as plot  # unconventional
import dask.array as darray  # unconventional
import dask.dataframe as ddf  # unconventional
import numpy as nmp  # unconventional
import pandas as pdas  # unconventional
import seaborn as sbrn  # unconventional
import tensorflow as tfz  # unconventional
import holoviews as hsv  # unconventional
import panel as pns  # unconventional
import plotly.express as pltx  # unconventional
import matplotlib as ml  # unconventional
import polars as ps # unconventional
import pyarrow as arr  # unconventional

import altair as alt  # conventional
import dask.array as da  # conventional
import dask.dataframe as dd  # conventional
import matplotlib.pyplot as plt  # conventional
import numpy as np  # conventional
import pandas as pd  # conventional
import seaborn as sns  # conventional
import tensorflow as tf # conventional
import holoviews as hv  # conventional
import panel as pn  # conventional
import plotly.express as px  # conventional
import matplotlib as mpl  # conventional
import polars as pl # conventional
import pyarrow as pa  # conventional

from tensorflow.keras import Model  # conventional



#: E701:1:5
if a: a = False
#: E701:1:40
if not header or header[:6] != 'bytes=': return
#: E702:1:10
a = False; b = True
#: E702:1:17
import bdist_egg; bdist_egg.write_safety_flag(cmd.egg_info, safe)
#: E703:1:13
import shlex;
#: E702:1:9 E703:1:23
del a[:]; a.append(42);
#: E704:1:1
def f(x): return 2
#: E704:1:1
async def f(x): return 2
#: E704:1:1 E271:1:6
async  def f(x): return 2
#: E704:1:1 E226:1:19
def f(x): return 2*x
#: E704:2:5 E226:2:23
while all is round:
    def f(x): return 2*x
#: E704:1:8 E702:1:11 E703:1:14
if True: x; y;
#: E701:1:8
if True: lambda a: b
#: E701:1:10
if a := 1: pass
# E701:1:4 E701:2:18 E701:3:8
try: lambda foo: bar
except ValueError: pass
finally: pass
# E701:1:7
class C: pass
# E701:1:7
with C(): pass
# E701:1:14
async with C(): pass
#:
lambda a: b
#:
a: List[str] = []
#:
if a := 1:
    pass
#:
func = lambda x: x** 2 if cond else lambda x:x
#:
class C: ...
#:
def f(): ...
#: E701:1:8 E702:1:13
class C: ...; x = 1
#: E701:1:8 E702:1:13
class C: ...; ...
#: E701:2:12
match *0, 1, *2:
    case 0,: y = 0
#:
class Foo:
    match: Optional[Match] = None
#: E702:2:4
while 1:
  1;...
#: E703:2:1
0\
;
#: E701:2:3
a = \
  5;



# Bad: This example returns 3!
def foo():
    try:
        1 / 0
        return "1"
    except:
        return "2"
    finally:
        return "3"


# Good
def foo():
    return_value = None
    try:
        1 / 0
        return_value = "1"
    except:
        return_value = "2"
    finally:
        return_value = "3"  # you might also want to check if "except" happened
    return return_value



def func(_, a, badAllowed):
    return _, a, badAllowed

def func(_, a, stillBad):
    return _, a, stillBad

class Class:
    def method(self, _, a, badAllowed):
        return _, a, badAllowed

    def method(self, _, a, stillBad):
        return _, a, stillBad



t = tuple()
l = list()
d1 = dict()
d2 = dict(a=1)
d3 = dict(**d2)


def list():
    return [1, 2, 3]


a = list()

f"{dict(x='y')}"
f'{dict(x="y")}'
f"{dict()}"
f"a {dict()} b"

f"{dict(x='y') | dict(y='z')}"
f"{ dict(x='y') | dict(y='z') }"
f"a {dict(x='y') | dict(y='z')} b"
f"a { dict(x='y') | dict(y='z') } b"



try:
    1 / 0
except ZeroDivisionError or ValueError as e:  # [binary-op-exception]
    pass

try:
    raise ValueError
except ZeroDivisionError and ValueError as e:  # [binary-op-exception]
    print(e)

try:
    pass
except (ValueError, Exception, IOError):
    pass



if True:
    import foo1; x = 1
    import foo2;     x = 1

if True:
    import foo3; \
x = 1

if True:
    import foo4 \
        ; x = 1

if True:
    x = 1; import foo5


if True:
    x = 1; \
         import foo6

if True:
    x = 1 \
        ; import foo7

if True:
    x = 1; import foo8; x = 1
    x = 1;     import foo9;     x = 1

if True:
    x = 1; \
        import foo10; \
    x = 1

if True:
    x = 1 \
        ;import foo11 \
        ;x = 1

if True:
    x = 1; \
        \
        import foo12

if True:
    x = 1; \
\
    import foo13


if True:
    x = 1; \
    #    \
    import foo14

# Continuation, but not as the last content in the file.
x = 1; \
import foo15

# Continuation, followed by end-of-file. (Removing `import foo` would cause a syntax
# error.)
x = 1; \
import foo16


#!/usr/bin/env python3
"""Generate boilerplate for a new Flake8 plugin.

Example usage:

    python scripts/add_plugin.py \
        flake8-pie \
        --url https://pypi.org/project/flake8-pie/ \
        --prefix PIE
"""
from __future__ import annotations

import argparse

from _utils import ROOT_DIR, dir_name, get_indent, pascal_case


def main(*, plugin: str, url: str, prefix_code: str) -> None:
    """Generate boilerplate for a new plugin."""
    # Create the test fixture folder.
    (ROOT_DIR / "crates/ruff_linter/resources/test/fixtures" / dir_name(plugin)).mkdir(
        exist_ok=True,
    )

    # Create the Plugin rules module.
    plugin_dir = ROOT_DIR / "crates/ruff_linter/src/rules" / dir_name(plugin)
    plugin_dir.mkdir(exist_ok=True)

    with (plugin_dir / "mod.rs").open("w+") as fp:
        fp.write(f"//! Rules from [{plugin}]({url}).\n")
        fp.write("pub(crate) mod rules;\n")
        fp.write("\n")
        fp.write(
            """#[cfg(test)]
mod tests {
    use std::convert::AsRef;
    use std::path::Path;

    use anyhow::Result;
    use test_case::test_case;

    use crate::registry::Rule;
    use crate::test::test_path;
    use crate::{assert_messages, settings};

    fn rules(rule_code: Rule, path: &Path) -> Result<()> {
        let snapshot = format!("{}_{}", rule_code.as_ref(), path.to_string_lossy());
        let diagnostics = test_path(
            Path::new("%s").join(path).as_path(),
            &settings::Settings::for_rule(rule_code),
        )?;
        assert_messages!(snapshot, diagnostics);
        Ok(())
    }
}
"""
            % dir_name(plugin),
        )

    # Create a subdirectory for rules and create a `mod.rs` placeholder
    rules_dir = plugin_dir / "rules"
    rules_dir.mkdir(exist_ok=True)

    (rules_dir / "mod.rs").touch()

    # Create the snapshots subdirectory
    (plugin_dir / "snapshots").mkdir(exist_ok=True)

    # Add the plugin to `rules/mod.rs`.
    rules_mod_path = ROOT_DIR / "crates/ruff_linter/src/rules/mod.rs"
    lines = rules_mod_path.read_text().strip().splitlines()
    lines.append(f"pub mod {dir_name(plugin)};")
    lines.sort()
    rules_mod_path.write_text("\n".join(lines) + "\n")

    # Add the relevant sections to `src/registry.rs`.
    content = (ROOT_DIR / "crates/ruff_linter/src/registry.rs").read_text()

    with (ROOT_DIR / "crates/ruff_linter/src/registry.rs").open("w") as fp:
        for line in content.splitlines():
            indent = get_indent(line)

            if line.strip() == "// ruff":
                fp.write(f"{indent}// {plugin}")
                fp.write("\n")

            elif line.strip() == "/// Ruff-specific rules":
                fp.write(f"{indent}/// [{plugin}]({url})\n")
                fp.write(f'{indent}#[prefix = "{prefix_code}"]\n')
                fp.write(f"{indent}{pascal_case(plugin)},")
                fp.write("\n")

            fp.write(line)
            fp.write("\n")

    text = ""
    with (ROOT_DIR / "crates/ruff_linter/src/codes.rs").open("r") as fp:
        while (line := next(fp)).strip() != "// ruff":
            text += line
        text += " " * 8 + f"// {plugin}\n\n"
        text += line
        text += fp.read()

    with (ROOT_DIR / "crates/ruff_linter/src/codes.rs").open("w") as fp:
        fp.write(text)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Generate boilerplate for a new Flake8 plugin.",
        epilog=(
            "Example usage: python scripts/add_plugin.py flake8-pie "
            "--url https://pypi.org/project/flake8-pie/"
        ),
    )
    parser.add_argument(
        "plugin",
        type=str,
        help="The name of the plugin to generate.",
    )
    parser.add_argument(
        "--url",
        required=True,
        type=str,
        help="The URL of the latest release in PyPI.",
    )
    parser.add_argument(
        "--prefix",
        required=False,
        default="TODO",
        type=str,
        help="Prefix code for the plugin. Leave empty to manually fill.",
    )
    args = parser.parse_args()

    main(plugin=args.plugin, url=args.url, prefix_code=args.prefix)



def f1(x: str = "50 character stringggggggggggggggggggggggggggggggg") -> None: ...  # OK
def f2(
    x: str = "51 character stringgggggggggggggggggggggggggggggggg",  # Error: PYI053
) -> None: ...
def f3(
    x: str = "50 character stringgggggggggggggggggggggggggggggg\U0001f600",  # OK
) -> None: ...
def f4(
    x: str = "51 character stringggggggggggggggggggggggggggggggg\U0001f600",  # Error: PYI053
) -> None: ...
def f5(
    x: bytes = b"50 character byte stringgggggggggggggggggggggggggg",  # OK
) -> None: ...
def f6(
    x: bytes = b"51 character byte stringgggggggggggggggggggggggggg",  # Error: PYI053
) -> None: ...
def f7(
    x: bytes = b"50 character byte stringggggggggggggggggggggggggg\xff",  # OK
) -> None: ...
def f8(
    x: bytes = b"51 character byte stringgggggggggggggggggggggggggg\xff",  # Error: PYI053
) -> None: ...

foo: str = "50 character stringggggggggggggggggggggggggggggggg"  # OK

bar: str = "51 character stringgggggggggggggggggggggggggggggggg"  # Error: PYI053

baz: bytes = b"50 character byte stringgggggggggggggggggggggggggg"  # OK

qux: bytes = b"51 character byte stringggggggggggggggggggggggggggg\xff"  # Error: PYI053

class Demo:
    """Docstrings are excluded from this rule. Some padding."""  # OK

def func() -> None:
    """Docstrings are excluded from this rule. Some padding."""  # OK



def func():
    pass

def func():
    pass

def func():
    return

def func():
    return 1

def func():
    return 1
    return "unreachable"

def func():
    i = 0

def func():
    i = 0
    i += 2
    return i



def f(a: int, b: int) -> int:
    """Showcase function.

    Parameters
    ----------
    a : int
        _description_
    b : int
        _description_
    Returns
    -------
    int
        _description
    """
    return b - a


def f() -> int:
    """Showcase function.

    Parameters
    ----------
    Returns
    -------
    """



import sys
from sys import platform, version_info

if sys.version_info[:1] == (2, 7): ...  # Y005
if sys.version_info[:2] == (2,): ...  # Y005


if sys.version_info[0] == 2: ...
if version_info[0] == 2: ...
if sys.version_info < (3, 5): ...
if version_info >= (3, 5): ...
if sys.version_info[:2] == (2, 7): ...
if sys.version_info[:1] == (2,): ...
if platform == 'linux': ...



"{}".format(1, bar=2)  # F522
"{bar}{}".format(1, bar=2, spam=3)  # F522
"{bar:{spam}}".format(bar=2, spam=3)  # No issues
"{bar:{spam}}".format(bar=2, spam=3, eggs=4, ham=5)  # F522
(''
 .format(x=2))  # F522



x = set(x for x in range(3))
x = set(x for x in range(3))
y = f"{set(a if a < 6 else 0  for a in range(3))}"
_ = "{}".format(set(a if a < 6 else 0 for a in range(3)))
print(f"Hello {set(a for a in range(3))} World")


def f(x):
    return x


print(f'Hello {set(a for a in "abc")} World')
print(f"Hello {set(a for a in 'abc')} World")
print(f"Hello {set(f(a) for a in 'abc')} World")
print(f"{set(a for a in 'abc') - set(a for a in 'ab')}")
print(f"{ set(a for a in 'abc') - set(a for a in 'ab') }")

# The fix generated for this diagnostic is incorrect, as we add additional space
# around the set comprehension.
print(f"{ {set(a for a in 'abc')} }")



# Errors.
foo == foo

foo != foo

foo > foo

foo >= foo

foo < foo

foo <= foo

foo is foo

foo is not foo

foo in foo

foo not in foo

id(foo) == id(foo)

len(foo) == len(foo)

# Non-errors.
"foo" == "foo"  # This is flagged by `comparison-of-constant` instead.

foo == "foo"

foo == bar

foo != bar

foo > bar

foo >= bar

foo < bar

foo <= bar

foo is bar

foo is not bar

foo in bar

foo not in bar

x(foo) == y(foo)

id(foo) == id(bar)

id(foo, bar) == id(foo, bar)

id(foo, bar=1) == id(foo, bar=1)



import abc
from abc import abstractmethod

import pytest


@pytest.fixture()
def my_fixture(mocker):  # OK with return
    return 0


@pytest.fixture()
def activate_context():  # OK with yield
    with get_context() as context:
        yield context


@pytest.fixture()
def _any_fixture(mocker):  # Ok nested function
    def nested_function():
        return 1

    mocker.patch("...", nested_function)


class BaseTest:
    @pytest.fixture()
    @abc.abstractmethod
    def _my_fixture():  # OK abstract with import abc
        return NotImplemented


class BaseTest:
    @pytest.fixture()
    @abstractmethod
    def _my_fixture():  # OK abstract with from import
        return NotImplemented


@pytest.fixture()
def _my_fixture(mocker):  # Error with return
    return 0


@pytest.fixture()
def _activate_context():  # Error with yield
    with get_context() as context:
        yield context


@pytest.fixture()
def _activate_context():  # Error with conditional yield from
    if some_condition:
        with get_context() as context:
            yield context
    else:
        yield from other_context()



import pytest


# OK
def f():
    pytest.fail("this is a failure")


def f():
    pytest.fail(msg="this is a failure")


def f():
    pytest.fail(reason="this is a failure")


# Errors
def f():
    pytest.fail()
    pytest.fail("")
    pytest.fail(f"")
    pytest.fail(msg="")
    pytest.fail(msg=f"")
    pytest.fail(reason="")
    pytest.fail(reason=f"")



from typing import List


def func():
    pass


nums = []
nums2 = []
nums3: list[int] = func()
nums4: List[int]


class C:
    def append(self, x):
        pass


# Errors.


# FURB113
nums.append(1)
nums.append(2)
pass


# FURB113
nums3.append(1)
nums3.append(2)
pass


# FURB113
nums4.append(1)
nums4.append(2)
pass


# FURB113
nums.append(1)
nums2.append(1)
nums.append(2)
nums.append(3)
pass


# FURB113
nums.append(1)
nums2.append(1)
nums.append(2)
# FURB113
nums3.append(1)
nums.append(3)
# FURB113
nums4.append(1)
nums4.append(2)
nums3.append(2)
pass

# FURB113
nums.append(1)
nums.append(2)
nums.append(3)


if True:
    # FURB113
    nums.append(1)
    nums.append(2)


if True:
    # FURB113
    nums.append(1)
    nums.append(2)
    pass


if True:
    # FURB113
    nums.append(1)
    nums2.append(1)
    nums.append(2)
    nums.append(3)


def yes_one(x: list[int]):
    # FURB113
    x.append(1)
    x.append(2)


def yes_two(x: List[int]):
    # FURB113
    x.append(1)
    x.append(2)


def yes_three(*, x: list[int]):
    # FURB113
    x.append(1)
    x.append(2)


def yes_four(x: list[int], /):
    # FURB113
    x.append(1)
    x.append(2)


def yes_five(x: list[int], y: list[int]):
    # FURB113
    x.append(1)
    x.append(2)
    y.append(1)
    x.append(3)


def yes_six(x: list):
    # FURB113
    x.append(1)
    x.append(2)


# Non-errors.

nums.append(1)
pass
nums.append(2)


if True:
    nums.append(1)
    pass
    nums.append(2)


nums.append(1)
pass


nums.append(1)
nums2.append(2)


nums.copy()
nums.copy()


c = C()
c.append(1)
c.append(2)


def not_one(x):
    x.append(1)
    x.append(2)


def not_two(x: C):
    x.append(1)
    x.append(2)


# redefining a list variable with a new type shouldn't confuse ruff.
nums2 = C()
nums2.append(1)
nums2.append(2)



from typing import ClassVar, Sequence, Final


class A:
    __slots__ = {
        "mutable_default": "A mutable default value",
    }

    mutable_default: list[int] = []
    immutable_annotation: Sequence[int] = []
    without_annotation = []
    class_variable: ClassVar[list[int]] = []
    final_variable: Final[list[int]] = []
    class_variable_without_subscript: ClassVar = []
    final_variable_without_subscript: Final = []


from dataclasses import dataclass, field


@dataclass
class C:
    mutable_default: list[int] = []
    immutable_annotation: Sequence[int] = []
    without_annotation = []
    perfectly_fine: list[int] = field(default_factory=list)
    class_variable: ClassVar[list[int]] = []
    final_variable: Final[list[int]] = []


from pydantic import BaseModel


class D(BaseModel):
    mutable_default: list[int] = []
    immutable_annotation: Sequence[int] = []
    without_annotation = []
    class_variable: ClassVar[list[int]] = []
    final_variable: Final[list[int]] = []



import os

import commands
import popen2

# Check all shell functions.
os.system("true")
os.popen("true")
os.popen2("true")
os.popen3("true")
os.popen4("true")
popen2.popen2("true")
popen2.popen3("true")
popen2.popen4("true")
popen2.Popen3("true")
popen2.Popen4("true")
commands.getoutput("true")
commands.getstatusoutput("true")


# Check command argument looks unsafe.
var_string = "true"
os.system(var_string)
os.system([var_string])
os.system([var_string, ""])



import mmap, select, socket
from mmap import error
# These should be fixed
try:
    pass
except EnvironmentError:
    pass

try:
    pass
except IOError:
    pass

try:
    pass
except WindowsError:
    pass

try:
    pass
except mmap.error:
    pass

try:
    pass
except select.error:
    pass

try:
    pass
except socket.error:
    pass

try:
    pass
except error:
    pass

# Should NOT be in parentheses when replaced

try:
    pass
except (IOError,):
    pass
try:
    pass
except (mmap.error,):
    pass
try:
    pass
except (EnvironmentError, IOError, OSError, select.error):
    pass

# Should be kept in parentheses (because multiple)

try:
    pass
except (IOError, KeyError, OSError):
    pass

# First should change, second should not
from .mmap import error
try:
    pass
except (IOError, error):
    pass
# These should not change

from foo import error

try:
    pass
except (OSError, error):
    pass

try:
    pass
except:
    pass

try:
    pass
except AssertionError:
    pass
try:
    pass
except (mmap).error:
    pass

try:
    pass
except OSError:
    pass

try:
    pass
except (OSError, KeyError):
    pass


# Regression test for: https://github.com/astral-sh/ruff/issues/7101
def get_owner_id_from_mac_address():
    try:
        mac_address = get_primary_mac_address()
    except(IOError, OSError) as ex:
        msg = 'Unable to query URL to get Owner ID: {u}\n{e}'.format(u=owner_id_url, e=ex)


# Regression test for: https://github.com/astral-sh/ruff/issues/7580
import os

try:
    pass
except os.error:
    pass



import contextlib
import pathlib
import pathlib as pl
from pathlib import Path
from pathlib import Path as P

# SIM115
f = open("foo.txt")
f = Path("foo.txt").open()
f = pathlib.Path("foo.txt").open()
f = pl.Path("foo.txt").open()
f = P("foo.txt").open()
data = f.read()
f.close()

# OK
with open("foo.txt") as f:
    data = f.read()

# OK
with contextlib.ExitStack() as exit_stack:
    f = exit_stack.enter_context(open("filename"))

# OK
with contextlib.ExitStack() as stack:
    files = [stack.enter_context(open(fname)) for fname in filenames]
    close_files = stack.pop_all().close

# OK
with contextlib.AsyncExitStack() as exit_stack:
    f = await exit_stack.enter_async_context(open("filename"))

# OK (false negative)
with contextlib.ExitStack():
    f = exit_stack.enter_context(open("filename"))

# SIM115
with contextlib.ExitStack():
    f = open("filename")

# OK
with contextlib.ExitStack() as exit_stack:
    exit_stack_ = exit_stack
    f = exit_stack_.enter_context(open("filename"))



import os
from os import sep


file_name = "foo/bar"

# PTH206
"foo/bar/".split(os.sep)
"foo/bar/".split(sep=os.sep)
"foo/bar/".split(os.sep)[-1]
"foo/bar/".split(os.sep)[-2]
"foo/bar/".split(os.sep)[-2:]
"fizz/buzz".split(sep)
"fizz/buzz".split(sep)[-1]
os.path.splitext("path/to/hello_world.py")[0].split(os.sep)[-1]
file_name.split(os.sep)
(os.path.abspath(file_name)).split(os.sep)

# OK
"foo/bar/".split("/")
"foo/bar/".split(os.sep, 1)
"foo/bar/".split(1, sep=os.sep)



def f1(x: str = "50 character stringggggggggggggggggggggggggggggggg") -> None:
    ...


def f2(x: str = "51 character stringgggggggggggggggggggggggggggggggg") -> None:
    ...


def f3(x: str = "50 character stringggggggggggggggggggggggggggggg\U0001f600") -> None:
    ...


def f4(x: str = "51 character stringgggggggggggggggggggggggggggggg\U0001f600") -> None:
    ...


def f5(x: bytes = b"50 character byte stringgggggggggggggggggggggggggg") -> None:
    ...


def f6(x: bytes = b"51 character byte stringgggggggggggggggggggggggggg") -> None:
    ...


def f7(x: bytes = b"50 character byte stringggggggggggggggggggggggggg\xff") -> None:
    ...


def f8(x: bytes = b"50 character byte stringgggggggggggggggggggggggggg\xff") -> None:
    ...


foo: str = "50 character stringggggggggggggggggggggggggggggggg"
bar: str = "51 character stringgggggggggggggggggggggggggggggggg"

baz: bytes = b"50 character byte stringgggggggggggggggggggggggggg"

qux: bytes = b"51 character byte stringggggggggggggggggggggggggggg\xff"


class Demo:
    """Docstrings are excluded from this rule. Some padding."""


def func() -> None:
    """Docstrings are excluded from this rule. Some padding."""



"""Update ruff.json in schemastore.

This script will clone astral-sh/schemastore, update the schema and push the changes
to a new branch tagged with the ruff git hash. You should see a URL to create the PR
to schemastore in the CLI.
"""
from __future__ import annotations

import json
from pathlib import Path
from subprocess import check_call, check_output
from tempfile import TemporaryDirectory

schemastore_fork = "git@github.com:astral-sh/schemastore.git"
schemastore_upstream = "git@github.com:SchemaStore/schemastore.git"
ruff_repo = "https://github.com/astral-sh/ruff"
root = Path(
    check_output(["git", "rev-parse", "--show-toplevel"], text=True).strip(),
)
ruff_json = Path("src/schemas/json/ruff.json")


def update_schemastore(schemastore: Path) -> None:
    if not schemastore.is_dir():
        check_call(["git", "clone", schemastore_fork, schemastore])
        check_call(
            [
                "git",
                "remote",
                "add",
                "upstream",
                schemastore_upstream,
            ],
            cwd=schemastore,
        )
    # Create a new branch tagged with the current ruff commit up to date with the latest
    # upstream schemastore
    check_call(["git", "fetch", "upstream"], cwd=schemastore)
    current_sha = check_output(["git", "rev-parse", "HEAD"], text=True).strip()
    branch = f"update-ruff-{current_sha}"
    check_call(
        ["git", "switch", "-c", branch],
        cwd=schemastore,
    )
    check_call(
        ["git", "reset", "--hard", "upstream/master"],
        cwd=schemastore,
    )

    # Update the schema and format appropriately
    schema = json.loads(root.joinpath("ruff.schema.json").read_text())
    schema["$id"] = "https://json.schemastore.org/ruff.json"
    schemastore.joinpath(ruff_json).write_text(
        json.dumps(dict(sorted(schema.items())), indent=2, ensure_ascii=False),
    )
    check_call(["prettier", "--write", ruff_json], cwd=schemastore)

    # Check if the schema has changed
    # https://stackoverflow.com/a/9393642/3549270
    if check_output(["git", "status", "-s"], cwd=schemastore).strip():
        # Schema has changed, commit and push
        commit_url = f"{ruff_repo}/commit/{current_sha}"
        commit_body = (
            f"This updates ruff's JSON schema to [{current_sha}]({commit_url})"
        )
        # https://stackoverflow.com/a/22909204/3549270
        check_call(
            [
                "git",
                "commit",
                "-a",
                "-m",
                "Update ruff's JSON schema",
                "-m",
                commit_body,
            ],
            cwd=schemastore,
        )
        # This should show the link to create a PR
        check_call(
            ["git", "push", "--set-upstream", "origin", branch],
            cwd=schemastore,
        )
    else:
        print("No changes")


def main() -> None:
    schemastore_existing = root.joinpath("schemastore")
    if schemastore_existing.is_dir():
        update_schemastore(schemastore_existing)
    else:
        with TemporaryDirectory() as temp_dir:
            update_schemastore(Path(temp_dir).joinpath("schemastore"))


if __name__ == "__main__":
    main()



"""A module docstring with D214 violations

    Returns
-----
    valid returns

    Args
-----
    valid args
"""

import os
from .expected import Expectation

expectation = Expectation()
expect = expectation.expect

expect(os.path.normcase(__file__ if __file__[-1] != 'c' else __file__[:-1]),
       "D214: Section is over-indented ('Returns')")
expect(os.path.normcase(__file__ if __file__[-1] != 'c' else __file__[:-1]),
       "D214: Section is over-indented ('Args')")


a = True if b else False  # SIM210

a = True if b != c else False  # SIM210

a = True if b + c else False  # SIM210

a = False if b else True  # OK


def f():
    # OK
    def bool():
        return False

    a = True if b else False


# Regression test for: https://github.com/astral-sh/ruff/issues/7076
samesld = True if (psl.privatesuffix(urlparse(response.url).netloc) ==
                                   psl.privatesuffix(src.netloc)) else False



# Errors

for item in {"apples", "lemons", "water"}:  # flags in-line set literals
    print(f"I like {item}.")

numbers_list = [i for i in {1, 2, 3}]  # flags sets in list comprehensions

numbers_set = {i for i in {1, 2, 3}}  # flags sets in set comprehensions

numbers_dict = {str(i): i for i in {1, 2, 3}}  # flags sets in dict comprehensions

numbers_gen = (i for i in {1, 2, 3})  # flags sets in generator expressions

# Non-errors

items = {"apples", "lemons", "water"}
for item in items:  # only complains about in-line sets (as per Pylint)
    print(f"I like {item}.")

for item in ["apples", "lemons", "water"]:  # lists are fine
    print(f"I like {item}.")

for item in ("apples", "lemons", "water"):  # tuples are fine
    print(f"I like {item}.")

numbers_list = [i for i in [1, 2, 3]]  # lists in comprehensions are fine

numbers_set = {i for i in (1, 2, 3)}  # tuples in comprehensions are fine

numbers_dict = {str(i): i for i in [1, 2, 3]}  # lists in dict comprehensions are fine

numbers_gen = (i for i in (1, 2, 3))  # tuples in generator expressions are fine

for item in set(("apples", "lemons", "water")):  # set constructor is fine
    print(f"I like {item}.")

for number in {i for i in range(10)}:  # set comprehensions are fine
    print(number)

for item in {*numbers_set, 4, 5, 6}:  # set unpacking is fine
    print(f"I like {item}.")



x = "𝐁ad string"
y = "−"


def f():
    """Here's a docstring with an unusual parenthesis: ）"""
    # And here's a comment with an unusual punctuation mark: ᜵
    ...


def f():
    """Here's a docstring with a greek rho: ρ"""
    # And here's a comment with a greek alpha: ∗
    ...


x = "𝐁ad string"
x = "−"

# This should be ignored, since it contains an unambiguous unicode character, and no
# ASCII.
x = "Русский"

# The first word should be ignored, while the second should be included, since it
# contains ASCII.
x = "βα Bαd"

# The two characters should be flagged here. The first character is a "word"
# consisting of a single ambiguous character, while the second character is a "word
# boundary" (whitespace) that it itself ambiguous.
x = "Р усский"



import typing
from typing import Optional
from typing import Union


def f(x: Optional[str]) -> None:
    ...


def f(x: typing.Optional[str]) -> None:
    ...


def f(x: Union[str, int, Union[float, bytes]]) -> None:
    ...


def f(x: typing.Union[str, int]) -> None:
    ...


def f(x: typing.Union[(str, int)]) -> None:
    ...


def f(x: typing.Union[(str, int), float]) -> None:
    ...


def f(x: typing.Union[(int,)]) -> None:
    ...


def f(x: typing.Union[()]) -> None:
    ...


def f(x: "Union[str, int, Union[float, bytes]]") -> None:
    ...


def f(x: "typing.Union[str, int]") -> None:
    ...


def f(x: Union["str", int]) -> None:
    ...


def f(x: Union[("str", "int"), float]) -> None:
    ...


def f() -> None:
    x: Optional[str]
    x = Optional[str]

    x = Union[str, int]
    x = Union["str", "int"]
    x: Union[str, int]
    x: Union["str", "int"]


def f(x: Union[int : float]) -> None:
    ...


def f(x: Union[str, int : float]) -> None:
    ...


def f(x: Union[x := int]) -> None:
    ...


def f(x: Union[str, x := int]) -> None:
    ...


def f(x: Union[lambda: int]) -> None:
    ...


def f(x: Union[str, lambda: int]) -> None:
    ...


def f(x: Optional[int : float]) -> None:
    ...


def f(x: Optional[str, int : float]) -> None:
    ...


def f(x: Optional[int, float]) -> None:
    ...


# Regression test for: https://github.com/astral-sh/ruff/issues/7131
class ServiceRefOrValue:
    service_specification: Optional[
        list[ServiceSpecificationRef]
        | list[ServiceSpecification]
    ] = None


# Regression test for: https://github.com/astral-sh/ruff/issues/7201
class ServiceRefOrValue:
    service_specification: Optional[str]is not True = None


# Regression test for: https://github.com/astral-sh/ruff/issues/7452
class Collection(Protocol[*_B0]):
    def __iter__(self) -> Iterator[Union[*_B0]]:
        ...



#!/usr/bin/env python3
"""Check code snippets in docs are formatted by black."""
from __future__ import annotations

import argparse
import os
import re
import textwrap
from pathlib import Path
from re import Match
from typing import TYPE_CHECKING

import black
from black.mode import Mode, TargetVersion
from black.parsing import InvalidInput

if TYPE_CHECKING:
    from collections.abc import Sequence

TARGET_VERSIONS = ["py37", "py38", "py39", "py310", "py311"]
SNIPPED_RE = re.compile(
    r"(?P<before>^(?P<indent> *)```\s*python\n)"
    r"(?P<code>.*?)"
    r"(?P<after>^(?P=indent)```\s*$)",
    re.DOTALL | re.MULTILINE,
)

# For some rules, we don't want black to fix the formatting as this would "fix" the
# example.
KNOWN_FORMATTING_VIOLATIONS = [
    "avoidable-escaped-quote",
    "bad-quotes-docstring",
    "bad-quotes-inline-string",
    "bad-quotes-multiline-string",
    "explicit-string-concatenation",
    "indent-with-spaces",
    "indentation-with-invalid-multiple",
    "line-too-long",
    "missing-trailing-comma",
    "missing-whitespace",
    "missing-whitespace-after-keyword",
    "missing-whitespace-around-arithmetic-operator",
    "missing-whitespace-around-bitwise-or-shift-operator",
    "missing-whitespace-around-modulo-operator",
    "missing-whitespace-around-operator",
    "missing-whitespace-around-parameter-equals",
    "multi-line-implicit-string-concatenation",
    "multiple-leading-hashes-for-block-comment",
    "multiple-spaces-after-comma",
    "multiple-spaces-after-keyword",
    "multiple-spaces-after-operator",
    "multiple-spaces-before-keyword",
    "multiple-spaces-before-operator",
    "multiple-statements-on-one-line-colon",
    "multiple-statements-on-one-line-semicolon",
    "no-blank-line-before-function",
    "no-indented-block-comment",
    "no-space-after-block-comment",
    "no-space-after-inline-comment",
    "non-empty-stub-body",
    "one-blank-line-after-class",
    "over-indentation",
    "over-indented",
    "pass-statement-stub-body",
    "prohibited-trailing-comma",
    "shebang-leading-whitespace",
    "surrounding-whitespace",
    "tab-indentation",
    "too-few-spaces-before-inline-comment",
    "trailing-comma-on-bare-tuple",
    "triple-single-quotes",
    "under-indentation",
    "unexpected-indentation-comment",
    "unexpected-spaces-around-keyword-parameter-equals",
    "unicode-kind-prefix",
    "unnecessary-class-parentheses",
    "useless-semicolon",
    "whitespace-after-open-bracket",
    "whitespace-before-close-bracket",
    "whitespace-before-parameters",
    "whitespace-before-punctuation",
]

# For some docs, black is unable to parse the example code.
KNOWN_PARSE_ERRORS = [
    "blank-line-with-whitespace",
    "indentation-with-invalid-multiple-comment",
    "missing-newline-at-end-of-file",
    "mixed-spaces-and-tabs",
    "no-indented-block",
    "non-pep695-type-alias",  # requires Python 3.12
    "syntax-error",
    "tab-after-comma",
    "tab-after-keyword",
    "tab-after-operator",
    "tab-before-keyword",
    "tab-before-operator",
    "trailing-whitespace",
    "unexpected-indentation",
]


class CodeBlockError(Exception):
    """A code block parse error."""


def format_str(
    src: str,
    black_mode: black.FileMode,
) -> tuple[str, Sequence[CodeBlockError]]:
    """Format a single docs file string."""
    errors: list[CodeBlockError] = []

    def _snipped_match(match: Match[str]) -> str:
        code = textwrap.dedent(match["code"])
        try:
            code = black.format_str(code, mode=black_mode)
        except InvalidInput as e:
            errors.append(CodeBlockError(e))

        code = textwrap.indent(code, match["indent"])
        return f'{match["before"]}{code}{match["after"]}'

    src = SNIPPED_RE.sub(_snipped_match, src)
    return src, errors


def format_file(
    file: Path,
    black_mode: black.FileMode,
    error_known: bool,
    args: argparse.Namespace,
) -> int:
    """Check the formatting of a single docs file.

    Returns the exit code for the script.
    """
    with file.open() as f:
        contents = f.read()

    if file.parent.name == "rules":
        # Check contents contains "What it does" section
        if "## What it does" not in contents:
            print(f"Docs for `{file.name}` are missing the `What it does` section.")
            return 1

        # Check contents contains "Why is this bad?" section
        if "## Why is this bad?" not in contents:
            print(f"Docs for `{file.name}` are missing the `Why is this bad?` section.")
            return 1

    # Remove everything before the first example
    contents = contents[contents.find("## Example") :]

    # Remove everything after the last example
    contents = contents[: contents.rfind("```")] + "```"

    new_contents, errors = format_str(contents, black_mode)

    if errors and not args.skip_errors and not error_known:
        for error in errors:
            rule_name = file.name.split(".")[0]
            print(
                f"Docs parse error for `{rule_name}` docs. Either fix or add to "
                f"`KNOWN_PARSE_ERRORS`. {error}",
            )

        return 2

    if contents != new_contents:
        rule_name = file.name.split(".")[0]
        print(
            f"Rule `{rule_name}` docs are not formatted. Either format the rule or add "
            f"to `KNOWN_FORMATTING_VIOLATIONS`. The example section should be "
            f"rewritten to:",
        )

        # Add indentation so that snipped can be copied directly to docs
        for line in new_contents.splitlines():
            output_line = "///"
            if len(line) > 0:
                output_line = f"{output_line} {line}"

            print(output_line)

        print("\n")

        return 1

    return 0


def main(argv: Sequence[str] | None = None) -> int:
    """Check code snippets in docs are formatted by black."""
    parser = argparse.ArgumentParser(
        description="Check code snippets in docs are formatted by black.",
    )
    parser.add_argument("--skip-errors", action="store_true")
    parser.add_argument("--generate-docs", action="store_true")
    args = parser.parse_args(argv)

    if args.generate_docs:
        # Generate docs
        from generate_mkdocs import main as generate_docs

        generate_docs()

    # Get static docs
    static_docs = [Path("docs") / f for f in os.listdir("docs") if f.endswith(".md")]

    # Check rules generated
    if not Path("docs/rules").exists():
        print("Please generate rules first.")
        return 1

    # Get generated rules
    generated_docs = [
        Path("docs/rules") / f for f in os.listdir("docs/rules") if f.endswith(".md")
    ]

    if len(generated_docs) == 0:
        print("Please generate rules first.")
        return 1

    black_mode = Mode(
        target_versions={TargetVersion[val.upper()] for val in TARGET_VERSIONS},
    )

    # Check known formatting violations and parse errors are sorted alphabetically and
    # have no duplicates. This will reduce the diff when adding new violations

    for known_list, file_string in [
        (KNOWN_FORMATTING_VIOLATIONS, "formatting violations"),
        (KNOWN_PARSE_ERRORS, "parse errors"),
    ]:
        if known_list != sorted(known_list):
            print(
                f"Known {file_string} is not sorted alphabetically. Please sort and "
                f"re-run.",
            )
            return 1

        duplicates = list({x for x in known_list if known_list.count(x) > 1})
        if len(duplicates) > 0:
            print(f"Known {file_string} has duplicates:")
            print("\n".join([f"  - {x}" for x in duplicates]))
            print("Please remove them and re-run.")
            return 1

    violations = 0
    errors = 0
    for file in [*static_docs, *generated_docs]:
        rule_name = file.name.split(".")[0]
        if rule_name in KNOWN_FORMATTING_VIOLATIONS:
            continue

        error_known = rule_name in KNOWN_PARSE_ERRORS

        result = format_file(file, black_mode, error_known, args)
        if result == 1:
            violations += 1
        elif result == 2 and not error_known:
            errors += 1

    if violations > 0:
        print(f"Formatting violations identified: {violations}")

    if errors > 0:
        print(f"New code block parse errors identified: {errors}")

    if violations > 0 or errors > 0:
        return 1

    return 0


if __name__ == "__main__":
    raise SystemExit(main())



from __future__ import annotations

from typing import (
    Annotated,
    Callable,
    List,
    Literal,
    NamedTuple,
    Tuple,
    TypeVar,
    TypedDict,
    cast,
)

from mypy_extensions import Arg, DefaultArg, DefaultNamedArg, NamedArg, VarArg


def foo(var: "MyClass") -> "MyClass":
    x: "MyClass"


def foo(*, inplace: "bool"):
    pass


def foo(*args: "str", **kwargs: "int"):
    pass


x: Tuple["MyClass"]

x: Callable[["MyClass"], None]


class Foo(NamedTuple):
    x: "MyClass"


class D(TypedDict):
    E: TypedDict("E", foo="int", total=False)


class D(TypedDict):
    E: TypedDict("E", {"foo": "int"})


x: Annotated["str", "metadata"]

x: Arg("str", "name")

x: DefaultArg("str", "name")

x: NamedArg("str", "name")

x: DefaultNamedArg("str", "name")

x: DefaultNamedArg("str", name="name")

x: VarArg("str")

x: List[List[List["MyClass"]]]

x: NamedTuple("X", [("foo", "int"), ("bar", "str")])

x: NamedTuple("X", fields=[("foo", "int"), ("bar", "str")])

x: NamedTuple(typename="X", fields=[("foo", "int")])

X: MyCallable("X")


# OK
class D(TypedDict):
    E: TypedDict("E")


x: Annotated[()]

x: DefaultNamedArg(name="name", quox="str")

x: DefaultNamedArg(name="name")

x: NamedTuple("X", [("foo",), ("bar",)])

x: NamedTuple("X", ["foo", "bar"])

x: NamedTuple()

x: Literal["foo", "bar"]

x = cast(x, "str")


def foo(x, *args, **kwargs):
    ...


def foo(*, inplace):
    ...


x: Annotated[1:2] = ...

x = TypeVar("x", "str", "int")

x = cast("str", x)

X = List["MyClass"]



import sys
from sys import version_info

if sys.version_info > (3, 5):
    3+6
else:
    3-5

if version_info > (3, 5):
    3+6
else:
    3-5

if sys.version_info >= (3,6):
    3+6
else:
    3-5

if version_info >= (3,6):
    3+6
else:
    3-5

if sys.version_info < (3,6):
    3-5
else:
    3+6

if sys.version_info <= (3,5):
    3-5
else:
    3+6

if sys.version_info <= (3, 5):
    3-5
else:
    3+6

if sys.version_info >= (3, 5):
    pass

if sys.version_info < (3,0):
    pass

if True:
    if sys.version_info < (3,0):
        pass

if sys.version_info < (3,0):
    pass
elif False:
    pass

if sys.version_info > (3,):
    pass
elif False:
    pass

if sys.version_info[0] > "2":
    3
else:
    2



class Person:  # [eq-without-hash]
    def __init__(self):
        self.name = "monty"

    def __eq__(self, other):
        return isinstance(other, Person) and other.name == self.name

# OK
class Language:
    def __init__(self):
        self.name = "python"

    def __eq__(self, other):
        return isinstance(other, Language) and other.name == self.name

    def __hash__(self):
        return hash(self.name)

class MyClass:
    def __eq__(self, other):
        return True

    __hash__ = None



x = [1, 2, 3]
y = [("a", 1), ("b", 2), ("c", 3)]
z = [(1,), (2,), (3,)]
d = {"a": 1, "b": 2, "c": 3}

[i for i in x]
{i for i in x}
{k: v for k, v in y}
{k: v for k, v in d.items()}
[(k, v) for k, v in d.items()]
{k: (a, b) for k, (a, b) in d.items()}

[i for i, in z]
[i for i, j in y]
[i for i in x if i > 1]
[i for i in x for j in x]

{v: k for k, v in y}
{k.foo: k for k in y}
{k["foo"]: k for k in y}
{k: v if v else None for k, v in y}

# Regression test for: https://github.com/astral-sh/ruff/issues/7196
any(len(symbol_table.get_by_type(symbol_type)) > 0 for symbol_type in[t for t in SymbolType])



from a import aksjdhflsakhdflkjsadlfajkslhfdkjsaldajlahflashdfljahlfksajlhfajfjfsaahflakjslhdfkjalhdskjfa
from a import aksjdhflsakhdflkjsadlfajkslhfdkjsaldajlahflashdfljahlfksajlhfajfjfsaahflakjslhdfkjalhdskjfa, aksjdhflsakhdflkjsadlfajkslhfdkjsaldajlahflashdfljahlfksajlhfajfjfsaahflakjslhdfkjalhdskjfa
from a import aksjdhflsakhdflkjsadlfajkslhfdkjsaldajlahflashdfljahlfksajlhfajfjfsaahflakjslhdfkjalhdskjfa as dfgsdfgsd, aksjdhflsakhdflkjsadlfajkslhfdkjsaldajlahflashdfljahlfksajlhfajfjfsaahflakjslhdfkjalhdskjfa as sdkjflsdjlahlfd

# At the top-level, force one empty line after an import, but allow up to two empty
# lines.
import os
import sys
x = 1

import os
import sys

x = 1

import os
import sys


x = 1

import os
import sys



x = 1


# In a nested scope, force one empty line after an import.
def func():
    import os
    import sys
    x = 1


def func():
    import os
    import sys

    x = 1


def func():
    import os
    import sys


    x = 1


def func():
    import os
    import sys



    x = 1



def func():
    async for i in range(5):
        print(i)

def func():
    async for i in range(20):
        print(i)
    else:
        return 0

def func():
    async for i in range(10):
        if i == 5:
            return 1
    return 0

def func():
    async for i in range(111):
        if i == 5:
            return 1
    else:
        return 0
    return 2

def func():
    async for i in range(12):
        continue

def func():
    async for i in range(1110):
        if True:
            continue

def func():
    async for i in range(13):
        break

def func():
    async for i in range(1110):
        if True:
            break



"""Test: noqa directives."""

from typing_extensions import List, Sequence

# This should ignore both errors.
from typing import (  # noqa: F811
    List,
    Sequence,
)

# This should ignore both errors.
from typing import (  # noqa
    List,
    Sequence,
)

# This should ignore both errors.
from typing import (
    List,  # noqa: F811
    Sequence,  # noqa: F811
)

# This should ignore both errors.
from typing import (
    List,  # noqa
    Sequence,  # noqa
)

# This should ignore the first error.
from typing import (
    List,  # noqa: F811
    Sequence,
)

# This should ignore both errors.
from typing import (  # noqa
    List,
    Sequence,
)

# This should ignore both errors.
from typing import List, Sequence  # noqa: F811

# This should ignore both errors.
from typing import List, Sequence  # noqa


def f():
    # This should ignore both errors.
    from typing import (  # noqa: F811
        List,
        Sequence,
    )



# ASCII literals should be replaced by a bytes literal
"foo".encode("utf-8")  # b"foo"
"foo".encode("u8")  # b"foo"
"foo".encode()  # b"foo"
"foo".encode("UTF8")  # b"foo"
U"foo".encode("utf-8")  # b"foo"
"foo".encode(encoding="utf-8")  # b"foo"
"""
Lorem

Ipsum
""".encode(
    "utf-8"
)
(
    "Lorem "
    "Ipsum".encode()
)
(
    "Lorem "  # Comment
    "Ipsum".encode()  # Comment
)
(
    "Lorem " "Ipsum".encode()
)

# `encode` on variables should not be processed.
string = "hello there"
string.encode("utf-8")

bar = "bar"
f"foo{bar}".encode("utf-8")
encoding = "latin"
"foo".encode(encoding)
f"foo{bar}".encode(encoding)
f"{a=} {b=}".encode(
    "utf-8",
)

# `encode` with custom args and kwargs should not be processed.
"foo".encode("utf-8", errors="replace")
"foo".encode("utf-8", "replace")
"foo".encode(errors="replace")
"foo".encode(encoding="utf-8", errors="replace")

# `encode` with custom args and kwargs on unicode should not be processed.
"unicode text©".encode("utf-8", errors="replace")
"unicode text©".encode("utf-8", "replace")
"unicode text©".encode(errors="replace")
"unicode text©".encode(encoding="utf-8", errors="replace")

# Unicode literals should only be stripped of default encoding.
"unicode text©".encode("utf-8")  # "unicode text©".encode()
"unicode text©".encode()
"unicode text©".encode(encoding="UTF8")  # "unicode text©".encode()

r"foo\o".encode("utf-8")  # br"foo\o"
u"foo".encode("utf-8")  # b"foo"
R"foo\o".encode("utf-8")  # br"foo\o"
U"foo".encode("utf-8")  # b"foo"
print("foo".encode())  # print(b"foo")

# `encode` on parenthesized strings.
(
    "abc"
    "def"
).encode()

((
    "abc"
    "def"
)).encode()

(f"foo{bar}").encode("utf-8")
(f"foo{bar}").encode(encoding="utf-8")
("unicode text©").encode("utf-8")
("unicode text©").encode(encoding="utf-8")


# Regression test for: https://github.com/astral-sh/ruff/issues/7455#issuecomment-1722459882
def _match_ignore(line):
	input=stdin and'\n'.encode()or None



import os

tempVar = os.getenv("TEST", 12)  # [invalid-envvar-default]
goodVar = os.getenv("TESTING", None)
dictVarBad = os.getenv("AAA", {"a", 7})  # [invalid-envvar-default]
print(os.getenv("TEST", False))  # [invalid-envvar-default]
os.getenv("AA", "GOOD")
os.getenv("AA", f"GOOD")
os.getenv("AA", "GOOD" + "BAD")
os.getenv("AA", "GOOD" + 1)
os.getenv("AA", "GOOD %s" % "BAD")
os.getenv("B", Z)




"""
Should emit:
B009 - Lines 19-31
B010 - Lines 40-45
"""

# Valid getattr usage
getattr(foo, bar)
getattr(foo, "bar", None)
getattr(foo, "bar{foo}".format(foo="a"), None)
getattr(foo, "bar{foo}".format(foo="a"))
getattr(foo, bar, None)
getattr(foo, "123abc")
getattr(foo, r"123\abc")
getattr(foo, "except")
getattr(foo, "__123abc")

# Invalid usage
getattr(foo, "bar")
getattr(foo, "_123abc")
getattr(foo, "__123abc__")
getattr(foo, "abc123")
getattr(foo, r"abc123")
_ = lambda x: getattr(x, "bar")
if getattr(x, "bar"):
    pass
getattr(1, "real")
getattr(1., "real")
getattr(1.0, "real")
getattr(1j, "real")
getattr(True, "real")
getattr(x := 1, "real")
getattr(x + y, "real")
getattr("foo"
        "bar", "real")


# Valid setattr usage
setattr(foo, bar, None)
setattr(foo, "bar{foo}".format(foo="a"), None)
setattr(foo, "123abc", None)
setattr(foo, "__123abc", None)
setattr(foo, r"123\abc", None)
setattr(foo, "except", None)
_ = lambda x: setattr(x, "bar", 1)
if setattr(x, "bar", 1):
    pass

# Invalid usage
setattr(foo, "bar", None)
setattr(foo, "_123abc", None)
setattr(foo, "__123abc__", None)
setattr(foo, "abc123", None)
setattr(foo, r"abc123", None)
setattr(foo.bar, r"baz", None)

# Regression test for: https://github.com/astral-sh/ruff/issues/7455#issuecomment-1722458885
assert getattr(func, '_rpc')is True



# UP034
print(("foo"))

# UP034
print(("hell((goodybe))o"))

# UP034
print((("foo")))

# UP034
print((((1))))

# UP034
print(("foo{}".format(1)))

# UP034
print(
    ("foo{}".format(1))
)

# UP034
print(
    (
        "foo"
    )
)

# UP034
def f():
    x = int(((yield 1)))

# UP034
if True:
    print(
        ("foo{}".format(1))
    )

# UP034
print((x for x in range(3)))

# OK
print("foo")

# OK
print((1, 2, 3))

# OK
print(())

# OK
print((1,))

# OK
sum((block.code for block in blocks), [])

# OK
def f():
    x = int((yield 1))

# OK
sum((i for i in range(3)), [])



# noqa
# noqa # comment
print()  # noqa
print()  # noqa # comment
print()  # noqa  # comment
print()  # noqa comment
print()  # noqa  comment
print(a)  # noqa
print(a)  # noqa # comment
print(a)  # noqa  # comment
print(a)  # noqa comment
print(a)  # noqa  comment

# noqa: E501, F821
# noqa: E501, F821 # comment
print()  # noqa: E501, F821
print()  # noqa: E501, F821 # comment
print()  # noqa: E501, F821  # comment
print()  # noqa: E501, F821 comment
print()  # noqa: E501, F821  comment
print(a)  # noqa: E501, F821
print(a)  # noqa: E501, F821 # comment
print(a)  # noqa: E501, F821  # comment
print(a)  # noqa: E501, F821 comment
print(a)  # noqa: E501, F821  comment



# Regression test: Don't forget the parentheses in the value when breaking
aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa: int = a + 1 * a


# Regression test: Don't forget the parentheses in the annotation when breaking
class DefaultRunner:
    task_runner_cls: TaskRunnerProtocol | typing.Callable[[], typing.Any] = DefaultTaskRunner



import subprocess


def foo():
    pass


# Errors.
subprocess.Popen(preexec_fn=foo)
subprocess.Popen(["ls"], preexec_fn=foo)
subprocess.Popen(preexec_fn=lambda: print("Hello, world!"))
subprocess.Popen(["ls"], preexec_fn=lambda: print("Hello, world!"))

# Non-errors.
subprocess.Popen()
subprocess.Popen(["ls"])
subprocess.Popen(preexec_fn=None)  # None is the default.
subprocess.Popen(["ls"], preexec_fn=None)  # None is the default.



from pathlib import Path, PurePath
from pathlib import Path as pth

# match
_ = Path(".")
_ = pth(".")
_ = PurePath(".")
_ = Path("")

# no match
_ = Path()
print(".")
Path("file.txt")
Path(".", "folder")
PurePath(".", "folder")



from typing import TypeVar, ParamSpec, NewType, TypeVarTuple

# Errors.

X = TypeVar("T")
X = TypeVar(name="T")

Y = ParamSpec("T")
Y = ParamSpec(name="T")

Z = NewType("T", int)
Z = NewType(name="T", tp=int)

Ws = TypeVarTuple("Ts")
Ws = TypeVarTuple(name="Ts")

# Non-errors.

T = TypeVar("T")
T = TypeVar(name="T")

T = ParamSpec("T")
T = ParamSpec(name="T")

T = NewType("T", int)
T = NewType(name="T", tp=int)

Ts = TypeVarTuple("Ts")
Ts = TypeVarTuple(name="Ts")

# Errors, but not covered by this rule.

# Non-string literal name.
T = TypeVar(some_str)
T = TypeVar(name=some_str)
T = TypeVar(1)
T = TypeVar(name=1)
T = ParamSpec(some_str)
T = ParamSpec(name=some_str)
T = ParamSpec(1)
T = ParamSpec(name=1)
T = NewType(some_str, int)
T = NewType(name=some_str, tp=int)
T = NewType(1, int)
T = NewType(name=1, tp=int)
Ts = TypeVarTuple(some_str)
Ts = TypeVarTuple(name=some_str)
Ts = TypeVarTuple(1)
Ts = TypeVarTuple(name=1)

# No names provided.
T = TypeVar()
T = ParamSpec()
T = NewType()
T = NewType(tp=int)
Ts = TypeVarTuple()



def f():
    # SIM110
    for x in iterable:
        if check(x):
            return True
    return False


def f():
    for x in iterable:
        if check(x):
            return True
    return True


def f():
    for el in [1, 2, 3]:
        if is_true(el):
            return True
    raise Exception


def f():
    # SIM111
    for x in iterable:
        if check(x):
            return False
    return True


def f():
    # SIM111
    for x in iterable:
        if not x.is_empty():
            return False
    return True


def f():
    for x in iterable:
        if check(x):
            return False
    return False


def f():
    for x in iterable:
        if check(x):
            return "foo"
    return "bar"


def f():
    # SIM110
    for x in iterable:
        if check(x):
            return True
    else:
        return False


def f():
    # SIM111
    for x in iterable:
        if check(x):
            return False
    else:
        return True


def f():
    # SIM110
    for x in iterable:
        if check(x):
            return True
    else:
        return False
    return True


def f():
    # SIM111
    for x in iterable:
        if check(x):
            return False
    else:
        return True
    return False


def f():
    for x in iterable:
        if check(x):
            return True
        elif x.is_empty():
            return True
    return False


def f():
    for x in iterable:
        if check(x):
            return True
        else:
            return True
    return False


def f():
    for x in iterable:
        if check(x):
            return True
        elif x.is_empty():
            return True
    else:
        return True
    return False


def f():
    def any(exp):
        pass

    for x in iterable:
        if check(x):
            return True
    return False


def f():
    def all(exp):
        pass

    for x in iterable:
        if check(x):
            return False
    return True


def f():
    x = 1

    # SIM110
    for x in iterable:
        if check(x):
            return True
    return False


def f():
    x = 1

    # SIM111
    for x in iterable:
        if check(x):
            return False
    return True


def f():
    # SIM110
    for x in "012ß9💣2ℝ9012ß9💣2ℝ9012ß9💣2ℝ9012ß9💣2ℝ9012ß9💣2ℝ":
        if x.isdigit():
            return True
    return False


def f():
    # OK (too long)
    for x in "012ß9💣2ℝ9012ß9💣2ℝ9012ß9💣2ℝ9012ß9💣2ℝ9012ß9💣2ℝ9":
        if x.isdigit():
            return True
    return False

async def f():
    # OK
    for x in iterable:
        if await check(x):
            return True
    return False

async def f():
    # SIM110
    for x in iterable:
        if check(x):
            return True
    return False



# Violations of PYI012

class OneAttributeClass:
    value: int
    pass  # PYI012 Class body must not contain `pass`

class OneAttributeClassRev:
    pass  # PYI012 Class body must not contain `pass`
    value: int

class DocstringClass:
    """
    My body only contains pass.
    """

    pass  # PYI012 Class body must not contain `pass`

class NonEmptyChild(Exception):
    value: int
    pass  # PYI012 Class body must not contain `pass`

class NonEmptyChild2(Exception):
    pass  # PYI012 Class body must not contain `pass`
    value: int

class NonEmptyWithInit:
    value: int
    pass  # PYI012 Class body must not contain `pass`

    def __init__():
        pass

# Not violations (of PYI012)

class EmptyClass:
    pass  # Y009 Empty body should contain `...`, not `pass`

class EmptyOneLine:
    pass  # Y009 Empty body should contain `...`, not `pass`

class Dog:
    eyes: int = 2

class EmptyEllipsis: ...

class NonEmptyEllipsis:
    value: int
    ...  # Y013 Non-empty class body must not contain `...`

class WithInit:
    value: int = 0

    def __init__():
        pass

def function():
    pass

pass



from typing import override


class Apples:
    def _init_(self):  # [bad-dunder-name]
        pass

    def __hello__(self):  # [bad-dunder-name]
        print("hello")

    def __init_(self):  # [bad-dunder-name]
        # author likely unintentionally misspelled the correct init dunder.
        pass

    def _init_(self):  # [bad-dunder-name]
        # author likely unintentionally misspelled the correct init dunder.
        pass

    def ___neg__(self):  # [bad-dunder-name]
        # author likely accidentally added an additional `_`
        pass

    def __inv__(self):  # [bad-dunder-name]
        # author likely meant to call the invert dunder method
        pass

    @override
    def _ignore__(self):  # [bad-dunder-name]
        # overridden dunder methods should be ignored
        pass

    def hello(self):
        print("hello")

    def __init__(self):
        pass

    def init(self):
        # valid name even though someone could accidentally mean __init__
        pass

    def _protected_method(self):
        print("Protected")

    def __private_method(self):
        print("Private")

    @property
    def __doc__(self):
        return "Docstring"

    # Allow dunder methods recommended by attrs.
    def __attrs_post_init__(self):
        pass

    def __attrs_pre_init__(self):
        pass

    def __attrs_init__(self):
        pass

    # Allow __html__, used by Jinja2 and Django.
    def __html__(self):
        pass


def __foo_bar__():  # this is not checked by the [bad-dunder-name] rule
    ...



class Everything:
    foo = 1

    def __init__(self):
        pass

    def _private(self):
        pass

    def method1(self):
        pass

    def method2(self):
        pass

    def method3(self):
        pass

    def method4(self):
        pass

    def method5(self):
        pass

    def method6(self):
        pass

    def method7(self):
        pass

    def method8(self):
        pass

    def method9(self):
        pass

class Small:
    def __init__(self):
        pass

    def _private(self):
        pass

    def method1(self):
        pass

    def method2(self):
        pass

    def method3(self):
        pass

    def method4(self):
        pass

    def method5(self):
        pass

    def method6(self):
        pass



dict([(i, i) for i in range(3)])
dict([(i, i) for i in range(3)], z=4)

def f(x):
    return x

f'{dict([(s,s) for s in "ab"])}'
f"{dict([(s,s) for s in 'ab'])}"
f"{dict([(s, s) for s in 'ab'])}"
f"{dict([(s,f(s)) for s in 'ab'])}"

f'{dict([(s,s) for s in "ab"]) | dict([(s,s) for s in "ab"])}'
f'{ dict([(s,s) for s in "ab"]) | dict([(s,s) for s in "ab"]) }'

# Regression test for: https://github.com/astral-sh/ruff/issues/7087
saved.append(dict([(k, v)for k,v in list(unique_instance.__dict__.items()) if k in [f.name for f in unique_instance._meta.fields]]))



import logging

logging.warning("Hello %s %s", "World!")  # [logging-too-few-args]

# do not handle calls with kwargs (like pylint)
logging.warning("Hello %s", "World!", "again", something="else")

logging.warning("Hello %s", "World!")

# do not handle calls without any args
logging.info("100% dynamic")

# do not handle calls with *args
logging.error("Example log %s, %s", "foo", "bar", "baz", *args)

# do not handle calls with **kwargs
logging.error("Example log %s, %s", "foo", "bar", "baz", **kwargs)

# do not handle keyword arguments
logging.error("%(objects)d modifications: %(modifications)d errors: %(errors)d")

logging.info(msg="Hello %s")

logging.info(msg="Hello %s %s")

import warning

warning.warning("Hello %s %s", "World!")



class C:
    badAllowed = 0
    stillBad = 0

    _badAllowed = 0
    _stillBad = 0

    bad_Allowed = 0
    still_Bad = 0

class D(TypedDict):
    badAllowed: bool
    stillBad: bool

    _badAllowed: list
    _stillBad: list

    bad_Allowed: set
    still_Bad: set



def after_return():
    return "reachable"
    return "unreachable"

async def also_works_on_async_functions():
    return "reachable"
    return "unreachable"

def if_always_true():
    if True:
        return "reachable"
    return "unreachable"

def if_always_false():
    if False:
        return "unreachable"
    return "reachable"

def if_elif_always_false():
    if False:
        return "unreachable"
    elif False:
        return "also unreachable"
    return "reachable"

def if_elif_always_true():
    if False:
        return "unreachable"
    elif True:
        return "reachable"
    return "also unreachable"

def ends_with_if():
    if False:
        return "unreachable"
    else:
        return "reachable"

def infinite_loop():
    while True:
        continue
    return "unreachable"

'''  TODO: we could determine these, but we don't yet.
def for_range_return():
    for i in range(10):
        if i == 5:
            return "reachable"
    return "unreachable"

def for_range_else():
    for i in range(111):
        if i == 5:
            return "reachable"
    else:
        return "unreachable"
    return "also unreachable"

def for_range_break():
    for i in range(13):
        return "reachable"
    return "unreachable"

def for_range_if_break():
    for i in range(1110):
        if True:
            return "reachable"
    return "unreachable"
'''

def match_wildcard(status):
    match status:
        case _:
            return "reachable"
    return "unreachable"

def match_case_and_wildcard(status):
    match status:
        case 1:
            return "reachable"
        case _:
            return "reachable"
    return "unreachable"

def raise_exception():
    raise Exception
    return "unreachable"

def while_false():
    while False:
        return "unreachable"
    return "reachable"

def while_false_else():
    while False:
        return "unreachable"
    else:
        return "reachable"

def while_false_else_return():
    while False:
        return "unreachable"
    else:
        return "reachable"
    return "also unreachable"

def while_true():
    while True:
        return "reachable"
    return "unreachable"

def while_true_else():
    while True:
        return "reachable"
    else:
        return "unreachable"

def while_true_else_return():
    while True:
        return "reachable"
    else:
        return "unreachable"
    return "also unreachable"

def while_false_var_i():
    i = 0
    while False:
        i += 1
    return i

def while_true_var_i():
    i = 0
    while True:
        i += 1
    return i

def while_infinite():
    while True:
        pass
    return "unreachable"

def while_if_true():
    while True:
        if True:
            return "reachable"
    return "unreachable"

# Test case found in the Bokeh repository that trigger a false positive.
def bokeh1(self, obj: BytesRep) -> bytes:
    data = obj["data"]

    if isinstance(data, str):
        return base64.b64decode(data)
    elif isinstance(data, Buffer):
        buffer = data
    else:
        id = data["id"]

        if id in self._buffers:
            buffer = self._buffers[id]
        else:
            self.error(f"can't resolve buffer '{id}'")

    return buffer.data

'''
TODO: because `try` statements aren't handled this triggers a false positive as
the last statement is reached, but the rules thinks it isn't (it doesn't
see/process the break statement).

# Test case found in the Bokeh repository that trigger a false positive.
def bokeh2(self, host: str = DEFAULT_HOST, port: int = DEFAULT_PORT) -> None:
    self.stop_serving = False
    while True:
        try:
            self.server = HTTPServer((host, port), HtmlOnlyHandler)
            self.host = host
            self.port = port
            break
        except OSError:
            log.debug(f"port {port} is in use, trying to next one")
            port += 1

    self.thread = threading.Thread(target=self._run_web_server)
'''



import sys

if sys.version_info < (3, 8):

    def a():
        if b:
            print(1)
        elif c:
            print(2)
        return None

else:
    pass


import sys

if sys.version_info < (3, 8):
    pass

else:

    def a():
        if b:
            print(1)
        elif c:
            print(2)
        else:
            print(3)
        return None




return len(self.nodeseeeeeeeee), sum(
    len(node.parents) for node in self.node_map.values()
)


return len(self.nodeseeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee), sum(
    len(node.parents) for node in self.node_map.values()
)


return (
    len(self.nodeseeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee), sum(
    len(node.parents) for node in self.node_map.values()
    )
)



type("")
type(b"")
type(0)
type(0.0)
type(0j)

# OK
type(arg)(" ")

# OK
y = x.dtype.type(0.0)

# Regression test for: https://github.com/astral-sh/ruff/issues/7455#issuecomment-1722459841
assert isinstance(fullname, type("")is not True)



from typing import ParamSpec, TypeVar

# Errors.

T = TypeVar("T", covariant=True, contravariant=True)
T = TypeVar(name="T", covariant=True, contravariant=True)

T = ParamSpec("T", covariant=True, contravariant=True)
T = ParamSpec(name="T", covariant=True, contravariant=True)

# Non-errors.

T = TypeVar("T")
T = TypeVar("T", covariant=False)
T = TypeVar("T", contravariant=False)
T = TypeVar("T", covariant=False, contravariant=False)
T = TypeVar("T", covariant=True)
T = TypeVar("T", covariant=True, contravariant=False)
T = TypeVar(name="T", covariant=True, contravariant=False)
T = TypeVar(name="T", covariant=True)
T = TypeVar("T", contravariant=True)
T = TypeVar("T", covariant=False, contravariant=True)
T = TypeVar(name="T", covariant=False, contravariant=True)
T = TypeVar(name="T", contravariant=True)

T = ParamSpec("T")
T = ParamSpec("T", covariant=False)
T = ParamSpec("T", contravariant=False)
T = ParamSpec("T", covariant=False, contravariant=False)
T = ParamSpec("T", covariant=True)
T = ParamSpec("T", covariant=True, contravariant=False)
T = ParamSpec(name="T", covariant=True, contravariant=False)
T = ParamSpec(name="T", covariant=True)
T = ParamSpec("T", contravariant=True)
T = ParamSpec("T", covariant=False, contravariant=True)
T = ParamSpec(name="T", covariant=False, contravariant=True)
T = ParamSpec(name="T", contravariant=True)



def bar():
    ...  # OK


def bar():
    pass  # OK


def bar():
    """oof"""  # OK


def oof():  # ERROR PYI048
    """oof"""
    print("foo")


def foo():  # ERROR PYI048
    """foo"""
    print("foo")
    print("foo")


def buzz():  # ERROR PYI048
    print("fizz")
    print("buzz")
    print("test")
